from dataclasses import dataclass
from datetime import time
from typing import Any, Literal, NamedTuple, NewType

from pydantic import BaseModel

RawPost = NewType("RawPost", Any)
Target = NewType("Target", str)
Category = int
Tag = str


@dataclass(eq=True, frozen=True)
class User:
    user: int
    user_type: Literal["group", "private"]


@dataclass(eq=True, frozen=True)
class PlatformTarget:
    target: str
    platform_name: str
    target_name: str


class UserSubInfo(NamedTuple):
    user: User
    categories: list[Category]
    tags: list[Tag]


class TimeWeightConfig(BaseModel):
    start_time: time
    end_time: time
    weight: int


class WeightConfig(BaseModel):
    default: int
    time_config: list[TimeWeightConfig]


class PlatformWeightConfigResp(BaseModel):
    target: Target
    target_name: str
    platform_name: str
    weight: WeightConfig



from .platform import platform_manager
from .scheduler import scheduler_dict
from .types import Target


async def check_sub_target(platform_name: str, target: Target):
    platform = platform_manager[platform_name]
    scheduler_conf_class = platform.scheduler
    scheduler = scheduler_dict[scheduler_conf_class]
    client = await scheduler.scheduler_config_obj.get_query_name_client()

    return await platform_manager[platform_name].get_target_name(client, target)



import asyncio
from logging.config import fileConfig

from alembic import context
from sqlalchemy import engine_from_config, pool
from sqlalchemy.engine.base import Connection

# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config

# Interpret the config file for Python logging.
# This line sets up loggers basically.
if config.config_file_name:
    fileConfig(config.config_file_name)  # type:ignore

# add your model's MetaData object here
# for 'autogenerate' support
# from myapp import mymodel
# target_metadata = mymodel.Base.metadata

import nonebot

try:
    nonebot.get_driver()
    __as_plugin = True
    target_metadata = None
except:
    __as_plugin = False
    nonebot.init()
    from nonebot_bison.config.db_model import Base

    target_metadata = Base.metadata

# other values from the config, defined by the needs of env.py,
# can be acquired:
# my_important_option = config.get_main_option("my_important_option")
# ... etc.


def run_migrations_offline():
    """Run migrations in 'offline' mode.

    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.

    Calls to context.execute() here emit the given string to the
    script output.

    """
    url = config.get_main_option("sqlalchemy.url")
    context.configure(
        url=url,
        target_metadata=target_metadata,
        literal_binds=True,
        dialect_opts={"paramstyle": "named"},
    )

    with context.begin_transaction():
        context.run_migrations()


def do_run_migration(connection: Connection):
    if __as_plugin:
        context.configure(connection=connection)
    else:
        context.configure(
            connection=connection,
            target_metadata=target_metadata,
            render_as_batch=True,
            compare_type=True,
        )

    with context.begin_transaction():
        context.run_migrations()


async def run_migrations_async():

    from nonebot_plugin_datastore.db import get_engine

    connectable = get_engine()
    async with connectable.connect() as connection:
        await connection.run_sync(do_run_migration)


def run_migrations_online():
    """Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.

    """
    if not __as_plugin:
        connectable = engine_from_config(
            config.get_section(config.config_ini_section),
            prefix="sqlalchemy.",
            poolclass=pool.NullPool,
        )

        with connectable.connect() as connection:
            do_run_migration(connection)
    else:
        # asyncio.run(run_migrations_async())
        asyncio.create_task(run_migrations_async())


if context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()



from abc import abstractmethod
from dataclasses import dataclass, field
from functools import reduce
from typing import Optional

from nonebot.adapters.onebot.v11.message import Message, MessageSegment

from ..plugin_config import plugin_config


@dataclass
class BasePost:
    @abstractmethod
    async def generate_text_messages(self) -> list[MessageSegment]:
        "Generate Message list from this instance"
        ...

    @abstractmethod
    async def generate_pic_messages(self) -> list[MessageSegment]:
        "Generate Message list from this instance with `use_pic`"
        ...


@dataclass
class OptionalMixin:
    # Because of https://stackoverflow.com/questions/51575931/class-inheritance-in-python-3-7-dataclasses

    override_use_pic: Optional[bool] = None
    compress: bool = False
    extra_msg: list[Message] = field(default_factory=list)

    def _use_pic(self):
        if not self.override_use_pic is None:
            return self.override_use_pic
        return plugin_config.bison_use_pic


@dataclass
class AbstractPost(OptionalMixin, BasePost):
    async def generate_messages(self) -> list[Message]:
        if self._use_pic():
            msg_segments = await self.generate_pic_messages()
        else:
            msg_segments = await self.generate_text_messages()
        if msg_segments:
            if self.compress:
                msgs = [reduce(lambda x, y: x.append(y), msg_segments, Message())]
            else:
                msgs = list(
                    map(lambda msg_segment: Message([msg_segment]), msg_segments)
                )
        else:
            msgs = []
        msgs.extend(self.extra_msg)
        return msgs



import datetime
import random
import string
from typing import Optional

import jwt

_key = "".join(random.SystemRandom().choice(string.ascii_letters) for _ in range(16))


def pack_jwt(obj: dict) -> str:
    return jwt.encode(
        {"exp": datetime.datetime.utcnow() + datetime.timedelta(hours=1), **obj},
        _key,
        algorithm="HS256",
    )


def load_jwt(token: str) -> Optional[dict]:
    try:
        return jwt.decode(token, _key, algorithms=["HS256"])
    except:
        return None



"""add time-weight table

Revision ID: 5f3370328e44
Revises: a333d6224193
Create Date: 2022-05-31 22:05:13.235981

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "5f3370328e44"
down_revision = "a333d6224193"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "schedule_time_weight",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("target_id", sa.Integer(), nullable=True),
        sa.Column("start_time", sa.Time(), nullable=True),
        sa.Column("end_time", sa.Time(), nullable=True),
        sa.Column("weight", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(
            ["target_id"],
            ["target.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    with op.batch_alter_table("target", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column("default_schedule_weight", sa.Integer(), nullable=True)
        )
        batch_op.drop_column("last_schedule_time")

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("target", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column("last_schedule_time", sa.DATETIME(), nullable=True)
        )
        batch_op.drop_column("default_schedule_weight")

    op.drop_table("schedule_time_weight")
    # ### end Alembic commands ###



from collections import defaultdict
from datetime import datetime, time
from typing import Awaitable, Callable, Optional

from nonebot_plugin_datastore.db import get_engine
from sqlalchemy.exc import IntegrityError
from sqlalchemy.ext.asyncio.session import AsyncSession
from sqlalchemy.orm import selectinload
from sqlalchemy.sql.expression import delete, select
from sqlalchemy.sql.functions import func

from ..types import Category, PlatformWeightConfigResp, Tag
from ..types import Target as T_Target
from ..types import TimeWeightConfig
from ..types import User as T_User
from ..types import UserSubInfo, WeightConfig
from .db_model import ScheduleTimeWeight, Subscribe, Target, User
from .utils import NoSuchTargetException


def _get_time():
    dt = datetime.now()
    cur_time = time(hour=dt.hour, minute=dt.minute, second=dt.second)
    return cur_time


class SubscribeDupException(Exception):
    ...


class DBConfig:
    def __init__(self):
        self.add_target_hook: Optional[Callable[[str, T_Target], Awaitable]] = None
        self.delete_target_hook: Optional[Callable[[str, T_Target], Awaitable]] = None

    def register_add_target_hook(self, fun: Callable[[str, T_Target], Awaitable]):
        self.add_target_hook = fun

    def register_delete_target_hook(self, fun: Callable[[str, T_Target], Awaitable]):
        self.delete_target_hook = fun

    async def add_subscribe(
        self,
        user: int,
        user_type: str,
        target: T_Target,
        target_name: str,
        platform_name: str,
        cats: list[Category],
        tags: list[Tag],
    ):
        async with AsyncSession(get_engine()) as session:
            db_user_stmt = (
                select(User).where(User.uid == user).where(User.type == user_type)
            )
            db_user: Optional[User] = await session.scalar(db_user_stmt)
            if not db_user:
                db_user = User(uid=user, type=user_type)
                session.add(db_user)
            db_target_stmt = (
                select(Target)
                .where(Target.platform_name == platform_name)
                .where(Target.target == target)
            )
            db_target: Optional[Target] = await session.scalar(db_target_stmt)
            if not db_target:
                db_target = Target(
                    target=target, platform_name=platform_name, target_name=target_name
                )
                if self.add_target_hook:
                    await self.add_target_hook(platform_name, target)
            else:
                db_target.target_name = target_name  # type: ignore
            subscribe = Subscribe(
                categories=cats,
                tags=tags,
                user=db_user,
                target=db_target,
            )
            session.add(subscribe)
            try:
                await session.commit()
            except IntegrityError as e:
                if len(e.args) > 0 and "UNIQUE constraint failed" in e.args[0]:
                    raise SubscribeDupException()
                raise e

    async def list_subscribe(self, user: int, user_type: str) -> list[Subscribe]:
        async with AsyncSession(get_engine()) as session:
            query_stmt = (
                select(Subscribe)
                .where(User.type == user_type, User.uid == user)
                .join(User)
                .options(selectinload(Subscribe.target))  # type:ignore
            )
            subs: list[Subscribe] = (await session.scalars(query_stmt)).all()
            return subs

    async def del_subscribe(
        self, user: int, user_type: str, target: str, platform_name: str
    ):
        async with AsyncSession(get_engine()) as session:
            user_obj = await session.scalar(
                select(User).where(User.uid == user, User.type == user_type)
            )
            target_obj = await session.scalar(
                select(Target).where(
                    Target.platform_name == platform_name, Target.target == target
                )
            )
            await session.execute(
                delete(Subscribe).where(
                    Subscribe.user == user_obj, Subscribe.target == target_obj
                )
            )
            target_count = await session.scalar(
                select(func.count())
                .select_from(Subscribe)
                .where(Subscribe.target == target_obj)
            )
            if target_count == 0:
                # delete empty target
                # await session.delete(target_obj)
                if self.delete_target_hook:
                    await self.delete_target_hook(platform_name, T_Target(target))
            await session.commit()

    async def update_subscribe(
        self,
        user: int,
        user_type: str,
        target: str,
        target_name: str,
        platform_name: str,
        cats: list,
        tags: list,
    ):
        async with AsyncSession(get_engine()) as sess:
            subscribe_obj: Subscribe = await sess.scalar(
                select(Subscribe)
                .where(
                    User.uid == user,
                    User.type == user_type,
                    Target.target == target,
                    Target.platform_name == platform_name,
                )
                .join(User)
                .join(Target)
                .options(selectinload(Subscribe.target))  # type:ignore
            )
            subscribe_obj.tags = tags  # type:ignore
            subscribe_obj.categories = cats  # type:ignore
            subscribe_obj.target.target_name = target_name
            await sess.commit()

    async def get_platform_target(self, platform_name: str) -> list[Target]:
        async with AsyncSession(get_engine()) as sess:
            subq = select(Subscribe.target_id).distinct().subquery()
            query = (
                select(Target).join(subq).where(Target.platform_name == platform_name)
            )
            return (await sess.scalars(query)).all()

    async def get_time_weight_config(
        self, target: T_Target, platform_name: str
    ) -> WeightConfig:
        async with AsyncSession(get_engine()) as sess:
            time_weight_conf: list[ScheduleTimeWeight] = (
                await sess.scalars(
                    select(ScheduleTimeWeight)
                    .where(
                        Target.platform_name == platform_name, Target.target == target
                    )
                    .join(Target)
                )
            ).all()
            targetObj: Target = await sess.scalar(
                select(Target).where(
                    Target.platform_name == platform_name, Target.target == target
                )
            )
            return WeightConfig(
                default=targetObj.default_schedule_weight,
                time_config=[
                    TimeWeightConfig(
                        start_time=time_conf.start_time,
                        end_time=time_conf.end_time,
                        weight=time_conf.weight,
                    )
                    for time_conf in time_weight_conf
                ],
            )

    async def update_time_weight_config(
        self, target: T_Target, platform_name: str, conf: WeightConfig
    ):
        async with AsyncSession(get_engine()) as sess:
            targetObj: Target = await sess.scalar(
                select(Target).where(
                    Target.platform_name == platform_name, Target.target == target
                )
            )
            if not targetObj:
                raise NoSuchTargetException()
            target_id = targetObj.id
            targetObj.default_schedule_weight = conf.default
            delete_statement = delete(ScheduleTimeWeight).where(
                ScheduleTimeWeight.target_id == target_id
            )
            await sess.execute(delete_statement)
            for time_conf in conf.time_config:
                new_conf = ScheduleTimeWeight(
                    start_time=time_conf.start_time,
                    end_time=time_conf.end_time,
                    weight=time_conf.weight,
                    target=targetObj,
                )
                sess.add(new_conf)

            await sess.commit()

    async def get_current_weight_val(self, platform_list: list[str]) -> dict[str, int]:
        res = {}
        cur_time = _get_time()
        async with AsyncSession(get_engine()) as sess:
            targets: list[Target] = (
                await sess.scalars(
                    select(Target)
                    .where(Target.platform_name.in_(platform_list))
                    .options(selectinload(Target.time_weight))
                )
            ).all()
            for target in targets:
                key = f"{target.platform_name}-{target.target}"
                weight = target.default_schedule_weight
                for time_conf in target.time_weight:
                    if (
                        time_conf.start_time <= cur_time
                        and time_conf.end_time > cur_time
                    ):
                        weight = time_conf.weight
                        break
                res[key] = weight
        return res

    async def get_platform_target_subscribers(
        self, platform_name: str, target: T_Target
    ) -> list[UserSubInfo]:
        async with AsyncSession(get_engine()) as sess:
            query = (
                select(Subscribe)
                .join(Target)
                .where(Target.platform_name == platform_name, Target.target == target)
                .options(selectinload(Subscribe.user))
            )
            subsribes: list[Subscribe] = (await sess.scalars(query)).all()
            return list(
                map(
                    lambda subscribe: UserSubInfo(
                        T_User(subscribe.user.uid, subscribe.user.type),
                        subscribe.categories,
                        subscribe.tags,
                    ),
                    subsribes,
                )
            )

    async def get_all_weight_config(
        self,
    ) -> dict[str, dict[str, PlatformWeightConfigResp]]:
        res: dict[str, dict[str, PlatformWeightConfigResp]] = defaultdict(dict)
        async with AsyncSession(get_engine()) as sess:
            query = select(Target)
            targets: list[Target] = (await sess.scalars(query)).all()
            query = select(ScheduleTimeWeight).options(
                selectinload(ScheduleTimeWeight.target)
            )
            time_weights: list[ScheduleTimeWeight] = (await sess.scalars(query)).all()

        for target in targets:
            platform_name = target.platform_name
            if platform_name not in res.keys():
                res[platform_name][target.target] = PlatformWeightConfigResp(
                    target=T_Target(target.target),
                    target_name=target.target_name,
                    platform_name=platform_name,
                    weight=WeightConfig(
                        default=target.default_schedule_weight, time_config=[]
                    ),
                )

        for time_weight_config in time_weights:
            platform_name = time_weight_config.target.platform_name
            target = time_weight_config.target.target
            res[platform_name][target].weight.time_config.append(
                TimeWeightConfig(
                    start_time=time_weight_config.start_time,
                    end_time=time_weight_config.end_time,
                    weight=time_weight_config.weight,
                )
            )
        return res


config = DBConfig()



import re
from typing import Any, Optional

from httpx import AsyncClient

from ..post import Post
from ..types import RawPost, Target
from ..utils import SchedulerConfig, http_client
from .platform import NewMessage


class NcmSchedConf(SchedulerConfig):

    name = "music.163.com"
    schedule_type = "interval"
    schedule_setting = {"minutes": 1}


class NcmArtist(NewMessage):

    categories = {}
    platform_name = "ncm-artist"
    enable_tag = False
    enabled = True
    is_common = True
    scheduler = NcmSchedConf
    name = "网易云-歌手"
    has_target = True
    parse_target_promot = "请输入歌手主页（包含数字ID）的链接"

    @classmethod
    async def get_target_name(
        cls, client: AsyncClient, target: Target
    ) -> Optional[str]:
        async with http_client() as client:
            res = await client.get(
                "https://music.163.com/api/artist/albums/{}".format(target),
                headers={"Referer": "https://music.163.com/"},
            )
            res_data = res.json()
            if res_data["code"] != 200:
                return
            return res_data["artist"]["name"]

    @classmethod
    async def parse_target(cls, target_text: str) -> Target:
        if re.match(r"^\d+$", target_text):
            return Target(target_text)
        elif match := re.match(
            r"(?:https?://)?music\.163\.com/#/artist\?id=(\d+)", target_text
        ):
            return Target(match.group(1))
        else:
            raise cls.ParseTargetException()

    async def get_sub_list(self, target: Target) -> list[RawPost]:
        async with http_client() as client:
            res = await client.get(
                "https://music.163.com/api/artist/albums/{}".format(target),
                headers={"Referer": "https://music.163.com/"},
            )
            res_data = res.json()
            if res_data["code"] != 200:
                return []
            else:
                return res_data["hotAlbums"]

    def get_id(self, post: RawPost) -> Any:
        return post["id"]

    def get_date(self, post: RawPost) -> int:
        return post["publishTime"] // 1000

    async def parse(self, raw_post: RawPost) -> Post:
        text = "新专辑发布：{}".format(raw_post["name"])
        target_name = raw_post["artist"]["name"]
        pics = [raw_post["picUrl"]]
        url = "https://music.163.com/#/album?id={}".format(raw_post["id"])
        return Post(
            "ncm-artist", text=text, url=url, pics=pics, target_name=target_name
        )


class NcmRadio(NewMessage):

    categories = {}
    platform_name = "ncm-radio"
    enable_tag = False
    enabled = True
    is_common = False
    scheduler = NcmSchedConf
    name = "网易云-电台"
    has_target = True
    parse_target_promot = "请输入主播电台主页（包含数字ID）的链接"

    @classmethod
    async def get_target_name(
        cls, client: AsyncClient, target: Target
    ) -> Optional[str]:
        async with http_client() as client:
            res = await client.post(
                "http://music.163.com/api/dj/program/byradio",
                headers={"Referer": "https://music.163.com/"},
                data={"radioId": target, "limit": 1000, "offset": 0},
            )
            res_data = res.json()
            if res_data["code"] != 200 or res_data["programs"] == 0:
                return
            return res_data["programs"][0]["radio"]["name"]

    @classmethod
    async def parse_target(cls, target_text: str) -> Target:
        if re.match(r"^\d+$", target_text):
            return Target(target_text)
        elif match := re.match(
            r"(?:https?://)?music\.163\.com/#/djradio\?id=(\d+)", target_text
        ):
            return Target(match.group(1))
        else:
            raise cls.ParseTargetException()

    async def get_sub_list(self, target: Target) -> list[RawPost]:
        async with http_client() as client:
            res = await client.post(
                "http://music.163.com/api/dj/program/byradio",
                headers={"Referer": "https://music.163.com/"},
                data={"radioId": target, "limit": 1000, "offset": 0},
            )
            res_data = res.json()
            if res_data["code"] != 200:
                return []
            else:
                return res_data["programs"]

    def get_id(self, post: RawPost) -> Any:
        return post["id"]

    def get_date(self, post: RawPost) -> int:
        return post["createTime"] // 1000

    async def parse(self, raw_post: RawPost) -> Post:
        text = "网易云电台更新：{}".format(raw_post["name"])
        target_name = raw_post["radio"]["name"]
        pics = [raw_post["coverUrl"]]
        url = "https://music.163.com/#/program/{}".format(raw_post["id"])
        return Post("ncm-radio", text=text, url=url, pics=pics, target_name=target_name)



from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship
from sqlalchemy.sql.schema import Column, ForeignKey, UniqueConstraint
from sqlalchemy.sql.sqltypes import JSON, Integer, String, Time

Base = declarative_base()


class User(Base):
    __tablename__ = "user"
    __table_args__ = (UniqueConstraint("type", "uid", name="unique-user-constraint"),)

    id = Column(Integer, primary_key=True, autoincrement=True)
    type = Column(String(20), nullable=False)
    uid = Column(Integer, nullable=False)

    subscribes = relationship("Subscribe", back_populates="user")


class Target(Base):
    __tablename__ = "target"
    __table_args__ = (
        UniqueConstraint("target", "platform_name", name="unique-target-constraint"),
    )

    id = Column(Integer, primary_key=True, autoincrement=True)
    platform_name = Column(String(20), nullable=False)
    target = Column(String(1024), nullable=False)
    target_name = Column(String(1024), nullable=False)
    default_schedule_weight = Column(Integer, default=10)

    subscribes = relationship("Subscribe", back_populates="target")
    time_weight = relationship("ScheduleTimeWeight", back_populates="target")


class ScheduleTimeWeight(Base):
    __tablename__ = "schedule_time_weight"

    id = Column(Integer, primary_key=True, autoincrement=True)
    target_id = Column(Integer, ForeignKey(Target.id))
    start_time = Column(Time)
    end_time = Column(Time)
    weight = Column(Integer)

    target = relationship("Target", back_populates="time_weight")


class Subscribe(Base):
    __tablename__ = "subscribe"
    __table_args__ = (
        UniqueConstraint("target_id", "user_id", name="unique-subscribe-constraint"),
    )

    id = Column(Integer, primary_key=True, autoincrement=True)
    target_id = Column(Integer, ForeignKey(Target.id))
    user_id = Column(Integer, ForeignKey(User.id))
    categories = Column(JSON)
    tags = Column(JSON)

    target = relationship("Target", back_populates="subscribes")
    user = relationship("User", back_populates="subscribes")



import functools

import httpx

from ..plugin_config import plugin_config

http_args = {
    "proxies": plugin_config.bison_proxy or None,
}
http_headers = {"user-agent": plugin_config.bison_ua}


@functools.wraps(httpx.AsyncClient)
def http_client(*args, **kwargs):
    if headers := kwargs.get("headers"):
        new_headers = http_headers.copy()
        new_headers.update(headers)
        kwargs["headers"] = new_headers
    else:
        kwargs["headers"] = http_headers
    return httpx.AsyncClient(*args, **kwargs)


http_client = functools.partial(http_client, **http_args)



import nonebot
from nonebot.adapters.onebot.v11.bot import Bot

from ..apis import check_sub_target
from ..config import (
    NoSuchSubscribeException,
    NoSuchTargetException,
    NoSuchUserException,
    config,
)
from ..config.db_config import SubscribeDupException
from ..platform import platform_manager
from ..types import Target as T_Target
from ..types import WeightConfig
from .jwt import pack_jwt
from .token_manager import token_manager


async def test():
    return {"status": 200, "text": "test"}


async def get_global_conf():
    res = {}
    for platform_name, platform in platform_manager.items():
        res[platform_name] = {
            "platformName": platform_name,
            "categories": platform.categories,
            "enabledTag": platform.enable_tag,
            "name": platform.name,
            "hasTarget": getattr(platform, "has_target"),
        }
    return {"platformConf": res}


async def get_admin_groups(qq: int):
    bot = nonebot.get_bot()
    groups = await bot.call_api("get_group_list")
    res = []
    for group in groups:
        group_id = group["group_id"]
        users = await bot.call_api("get_group_member_list", group_id=group_id)
        for user in users:
            if user["user_id"] == qq and user["role"] in ("owner", "admin"):
                res.append({"id": group_id, "name": group["group_name"]})
    return res


async def auth(token: str):
    if qq_tuple := token_manager.get_user(token):
        qq, nickname = qq_tuple
        bot = nonebot.get_bot()
        assert isinstance(bot, Bot)
        groups = await bot.call_api("get_group_list")
        if str(qq) in nonebot.get_driver().config.superusers:
            jwt_obj = {
                "id": qq,
                "type": "admin",
                "groups": list(
                    map(
                        lambda info: {
                            "id": info["group_id"],
                            "name": info["group_name"],
                        },
                        groups,
                    )
                ),
            }
            ret_obj = {
                "type": "admin",
                "name": nickname,
                "id": qq,
                "token": pack_jwt(jwt_obj),
            }
            return {"status": 200, **ret_obj}
        if admin_groups := await get_admin_groups(int(qq)):
            jwt_obj = {"id": str(qq), "type": "user", "groups": admin_groups}
            ret_obj = {
                "type": "user",
                "name": nickname,
                "id": qq,
                "token": pack_jwt(jwt_obj),
            }
            return {"status": 200, **ret_obj}
        else:
            return {"status": 400, "type": "", "name": "", "id": 0, "token": ""}
    else:
        return {"status": 400, "type": "", "name": "", "id": 0, "token": ""}


async def get_subs_info(jwt_obj: dict):
    groups = jwt_obj["groups"]
    res = {}
    for group in groups:
        group_id = group["id"]
        raw_subs = await config.list_subscribe(group_id, "group")
        subs = list(
            map(
                lambda sub: {
                    "platformName": sub.target.platform_name,
                    "targetName": sub.target.target_name,
                    "cats": sub.categories,
                    "tags": sub.tags,
                    "target": sub.target.target,
                },
                raw_subs,
            )
        )
        res[group_id] = {"name": group["name"], "subscribes": subs}
    return res


async def get_target_name(platform_name: str, target: str, jwt_obj: dict):
    return {"targetName": await check_sub_target(platform_name, target)}


async def add_group_sub(
    group_number: int,
    platform_name: str,
    target: str,
    target_name: str,
    cats: list[int],
    tags: list[str],
):
    try:
        await config.add_subscribe(
            int(group_number),
            "group",
            T_Target(target),
            target_name,
            platform_name,
            cats,
            tags,
        )
        return {"status": 200, "msg": ""}
    except SubscribeDupException:
        return {"status": 403, "msg": ""}


async def del_group_sub(group_number: int, platform_name: str, target: str):
    try:
        await config.del_subscribe(int(group_number), "group", target, platform_name)
    except (NoSuchUserException, NoSuchSubscribeException):
        return {"status": 400, "msg": "删除错误"}
    return {"status": 200, "msg": ""}


async def update_group_sub(
    group_number: int,
    platform_name: str,
    target: str,
    target_name: str,
    cats: list[int],
    tags: list[str],
):
    try:
        await config.update_subscribe(
            int(group_number), "group", target, target_name, platform_name, cats, tags
        )
    except (NoSuchUserException, NoSuchSubscribeException):
        return {"status": 400, "msg": "更新错误"}
    return {"status": 200, "msg": ""}


async def get_weight_config():
    return await config.get_all_weight_config()


async def update_weigth_config(
    platform_name: str, target: str, weight_config: WeightConfig
):
    try:
        await config.update_time_weight_config(
            T_Target(target), platform_name, weight_config
        )
    except NoSuchTargetException:
        return {"status": 400, "msg": "该订阅不存在"}
    return {"status": 200, "msg": ""}



from nonebot.plugin import require

from . import (
    admin_page,
    bootstrap,
    config,
    config_manager,
    platform,
    post,
    scheduler,
    send,
    types,
    utils,
)
from .plugin_config import plugin_config

require("nonebot_plugin_localstore")

__help__version__ = "0.4.3"
__help__plugin__name__ = "nonebot_bison"
__usage__ = f"本bot可以提供b站、微博等社交媒体的消息订阅，详情请查看本bot文档，或者{'at本bot' if plugin_config.bison_to_me else '' }发送“添加订阅”订阅第一个帐号，发送“查询订阅”或“删除订阅”管理订阅"

__all__ = [
    "admin_page",
    "config",
    "config_manager",
    "post",
    "scheduler",
    "send",
    "platform",
    "types",
    "utils",
]



"""add last scheduled time

Revision ID: a333d6224193
Revises: 4a46ba54a3f3
Create Date: 2022-03-29 21:01:38.213153

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "a333d6224193"
down_revision = "4a46ba54a3f3"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("target", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column("last_schedule_time", sa.DateTime(timezone=True), nullable=True)
        )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("target", schema=None) as batch_op:
        batch_op.drop_column("last_schedule_time")

    # ### end Alembic commands ###



import re
import sys
from typing import Union

import nonebot
from bs4 import BeautifulSoup as bs
from nonebot.adapters.onebot.v11.message import MessageSegment
from nonebot.log import default_format, logger
from nonebot.plugin import require

from ..plugin_config import plugin_config
from .http import http_client
from .scheduler_config import SchedulerConfig, scheduler

__all__ = [
    "http_client",
    "Singleton",
    "parse_text",
    "html_to_text",
    "SchedulerConfig",
    "scheduler",
]


class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


async def parse_text(text: str) -> MessageSegment:
    "return raw text if don't use pic, otherwise return rendered opcode"
    if plugin_config.bison_use_pic:
        require("nonebot_plugin_htmlrender")
        from nonebot_plugin_htmlrender import text_to_pic as _text_to_pic

        return MessageSegment.image(await _text_to_pic(text))
    else:
        return MessageSegment.text(text)


if not plugin_config.bison_skip_browser_check:
    require("nonebot_plugin_htmlrender")


def html_to_text(html: str, query_dict: dict = {}) -> str:
    html = re.sub(r"<br\s*/?>", "<br>\n", html)
    html = html.replace("</p>", "</p>\n")
    soup = bs(html, "html.parser")
    if query_dict:
        node = soup.find(**query_dict)
    else:
        node = soup
    assert node is not None
    return node.text.strip()


class Filter:
    def __init__(self) -> None:
        self.level: Union[int, str] = "DEBUG"

    def __call__(self, record):
        module_name: str = record["name"]
        module = sys.modules.get(module_name)
        if module:
            module_name = getattr(module, "__module_name__", module_name)
        record["name"] = module_name.split(".")[0]
        levelno = (
            logger.level(self.level).no if isinstance(self.level, str) else self.level
        )
        nonebot_warning_level = logger.level("WARNING").no
        return (
            record["level"].no >= levelno
            if record["name"] != "nonebot"
            else record["level"].no >= nonebot_warning_level
        )


if plugin_config.bison_filter_log:
    logger.remove()
    default_filter = Filter()
    logger.add(
        sys.stdout,
        colorize=True,
        diagnose=False,
        filter=default_filter,
        format=default_format,
    )
    config = nonebot.get_driver().config
    logger.success("Muted info & success from nonebot")
    default_filter.level = (
        ("DEBUG" if config.debug else "INFO")
        if config.log_level is None
        else config.log_level
    )



from typing import Optional

import nonebot
from pydantic import BaseSettings


class PlugConfig(BaseSettings):

    bison_config_path: str = ""
    bison_use_pic: bool = False
    bison_init_filter: bool = True
    bison_use_queue: bool = True
    bison_outer_url: str = "http://localhost:8080/bison/"
    bison_filter_log: bool = False
    bison_to_me: bool = True
    bison_skip_browser_check: bool = False
    bison_use_pic_merge: int = 0  # 多图片时启用图片合并转发（仅限群）
    # 0：不启用；1：首条消息单独发送，剩余照片合并转发；2以及以上：所有消息全部合并转发
    bison_resend_times: int = 0
    bison_proxy: Optional[str]
    bison_ua: str = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36"

    class Config:
        extra = "ignore"


global_config = nonebot.get_driver().config
plugin_config = PlugConfig(**global_config.dict())



import random
import string
from typing import Optional

from expiringdict import ExpiringDict


class TokenManager:
    def __init__(self):
        self.token_manager = ExpiringDict(max_len=100, max_age_seconds=60 * 10)

    def get_user(self, token: str) -> Optional[tuple]:
        res = self.token_manager.get(token)
        assert res is None or isinstance(res, tuple)
        return res

    def save_user(self, token: str, qq: tuple) -> None:
        self.token_manager[token] = qq

    def get_user_token(self, qq: tuple) -> str:
        token = "".join(random.choices(string.ascii_letters + string.digits, k=16))
        self.save_user(token, qq)
        return token


token_manager = TokenManager()



import time
from typing import Literal, Union

from nonebot.adapters.onebot.v11.bot import Bot
from nonebot.adapters.onebot.v11.message import Message, MessageSegment
from nonebot.log import logger

from .plugin_config import plugin_config

QUEUE: list[
    tuple[
        Bot,
        int,
        Literal["private", "group", "group-forward"],
        Union[str, Message],
        int,
    ]
] = []
LAST_SEND_TIME = time.time()


async def _do_send(
    bot: "Bot",
    user: int,
    user_type: Literal["group", "private", "group-forward"],
    msg: Union[str, Message],
):
    if user_type == "group":
        await bot.send_group_msg(group_id=user, message=msg)
    elif user_type == "private":
        await bot.send_private_msg(user_id=user, message=msg)
    elif user_type == "group-forward":
        await bot.send_group_forward_msg(group_id=user, messages=msg)


async def do_send_msgs():
    global LAST_SEND_TIME
    if time.time() - LAST_SEND_TIME < 1.5:
        return
    if QUEUE:
        bot, user, user_type, msg, retry_time = QUEUE.pop(0)
        try:
            await _do_send(bot, user, user_type, msg)
        except Exception as e:
            if retry_time > 0:
                QUEUE.insert(0, (bot, user, user_type, msg, retry_time - 1))
            else:
                msg_str = str(msg)
                if len(msg_str) > 50:
                    msg_str = msg_str[:50] + "..."
                logger.warning(f"send msg err {e} {msg_str}")
        LAST_SEND_TIME = time.time()


async def _send_msgs_dispatch(
    bot: Bot,
    user,
    user_type: Literal["private", "group", "group-forward"],
    msg: Union[str, Message],
):
    if plugin_config.bison_use_queue:
        QUEUE.append((bot, user, user_type, msg, plugin_config.bison_resend_times))
    else:
        await _do_send(bot, user, user_type, msg)


async def send_msgs(
    bot: Bot, user, user_type: Literal["private", "group"], msgs: list[Message]
):
    if not plugin_config.bison_use_pic_merge or user_type == "private":
        for msg in msgs:
            await _send_msgs_dispatch(bot, user, user_type, msg)
        return
    msgs = msgs.copy()
    if plugin_config.bison_use_pic_merge == 1:
        await _send_msgs_dispatch(bot, user, "group", msgs.pop(0))
    if msgs:
        if len(msgs) == 1:  # 只有一条消息序列就不合并转发
            await _send_msgs_dispatch(bot, user, "group", msgs.pop(0))
        else:
            group_bot_info = await bot.get_group_member_info(
                group_id=user, user_id=int(bot.self_id), no_cache=True
            )  # 调用api获取群内bot的相关参数
            # forward_msg = Message(
            #     [
            #         MessageSegment.node_custom(
            #             group_bot_info["user_id"],
            #             nickname=group_bot_info["card"] or group_bot_info["nickname"],
            #             content=msg,
            #         )
            #         for msg in msgs
            #     ]
            # )
            # FIXME: Because of https://github.com/nonebot/adapter-onebot/issues/9

            forward_msg = [
                {
                    "type": "node",
                    "data": {
                        "name": group_bot_info["card"] or group_bot_info["nickname"],
                        "uin": group_bot_info["user_id"],
                        "content": msg,
                    },
                }
                for msg in msgs
            ]

            await _send_msgs_dispatch(bot, user, "group-forward", forward_msg)



import functools
import json
import re
from copy import deepcopy
from dataclasses import dataclass, field
from datetime import datetime, timedelta
from typing import Any, Callable, Optional

import httpx
from httpx import AsyncClient
from nonebot.log import logger
from typing_extensions import Self

from ..plugin_config import plugin_config
from ..post import Post
from ..types import Category, RawPost, Tag, Target
from ..utils import SchedulerConfig
from ..utils.http import http_args
from .platform import CategoryNotSupport, NewMessage, StatusChange


class BilibiliSchedConf(SchedulerConfig):

    name = "bilibili.com"
    schedule_type = "interval"
    schedule_setting = {"seconds": 10}

    _client_refresh_time: datetime
    cookie_expire_time = timedelta(hours=5)

    def __init__(self):
        self._client_refresh_time = datetime(
            year=2000, month=1, day=1
        )  # an expired time
        super().__init__()

    async def _init_session(self):
        res = await self.default_http_client.get("https://www.bilibili.com/")
        if res.status_code != 200:
            logger.warning("unable to refresh temp cookie")
        else:
            self._client_refresh_time = datetime.now()

    async def _refresh_client(self):
        if datetime.now() - self._client_refresh_time > self.cookie_expire_time:
            await self._init_session()

    async def get_client(self, target: Target) -> AsyncClient:
        await self._refresh_client()
        return await super().get_client(target)

    async def get_query_name_client(self) -> AsyncClient:
        await self._refresh_client()
        return await super().get_query_name_client()


class Bilibili(NewMessage):

    categories = {
        1: "一般动态",
        2: "专栏文章",
        3: "视频",
        4: "纯文字",
        5: "转发"
        # 5: "短视频"
    }
    platform_name = "bilibili"
    enable_tag = True
    enabled = True
    is_common = True
    scheduler = BilibiliSchedConf
    name = "B站"
    has_target = True
    parse_target_promot = "请输入用户主页的链接"

    @classmethod
    async def get_target_name(
        cls, client: AsyncClient, target: Target
    ) -> Optional[str]:
        res = await client.get(
            "https://api.bilibili.com/x/space/acc/info", params={"mid": target}
        )
        res_data = json.loads(res.text)
        if res_data["code"]:
            return None
        return res_data["data"]["name"]

    @classmethod
    async def parse_target(cls, target_text: str) -> Target:
        if re.match(r"\d+", target_text):
            return Target(target_text)
        elif m := re.match(r"(?:https?://)?space\.bilibili\.com/(\d+)", target_text):
            return Target(m.group(1))
        else:
            raise cls.ParseTargetException()

    async def get_sub_list(self, target: Target) -> list[RawPost]:
        params = {"host_uid": target, "offset": 0, "need_top": 0}
        res = await self.client.get(
            "https://api.vc.bilibili.com/dynamic_svr/v1/dynamic_svr/space_history",
            params=params,
            timeout=4.0,
        )
        res_dict = json.loads(res.text)
        if res_dict["code"] == 0:
            return res_dict["data"].get("cards")
        else:
            return []

    def get_id(self, post: RawPost) -> Any:
        return post["desc"]["dynamic_id"]

    def get_date(self, post: RawPost) -> int:
        return post["desc"]["timestamp"]

    def _do_get_category(self, post_type: int) -> Category:
        if post_type == 2:
            return Category(1)
        elif post_type == 64:
            return Category(2)
        elif post_type == 8:
            return Category(3)
        elif post_type == 4:
            return Category(4)
        elif post_type == 1:
            # 转发
            return Category(5)
        raise CategoryNotSupport()

    def get_category(self, post: RawPost) -> Category:
        post_type = post["desc"]["type"]
        return self._do_get_category(post_type)

    def get_tags(self, raw_post: RawPost) -> list[Tag]:
        return [
            *map(
                lambda tp: tp["topic_name"],
                raw_post["display"]["topic_info"]["topic_details"],
            )
        ]

    def _get_info(self, post_type: Category, card) -> tuple[str, list]:
        if post_type == 1:
            # 一般动态
            text = card["item"]["description"]
            pic = [img["img_src"] for img in card["item"]["pictures"]]
        elif post_type == 2:
            # 专栏文章
            text = "{} {}".format(card["title"], card["summary"])
            pic = card["image_urls"]
        elif post_type == 3:
            # 视频
            text = card["dynamic"]
            pic = [card["pic"]]
        elif post_type == 4:
            # 纯文字
            text = card["item"]["content"]
            pic = []
        else:
            raise CategoryNotSupport()
        return text, pic

    async def parse(self, raw_post: RawPost) -> Post:
        card_content = json.loads(raw_post["card"])
        post_type = self.get_category(raw_post)
        target_name = raw_post["desc"]["user_profile"]["info"]["uname"]
        if post_type >= 1 and post_type < 5:
            url = ""
            if post_type == 1:
                # 一般动态
                url = "https://t.bilibili.com/{}".format(
                    raw_post["desc"]["dynamic_id_str"]
                )
            elif post_type == 2:
                # 专栏文章
                url = "https://www.bilibili.com/read/cv{}".format(
                    raw_post["desc"]["rid"]
                )
            elif post_type == 3:
                # 视频
                url = "https://www.bilibili.com/video/{}".format(
                    raw_post["desc"]["bvid"]
                )
            elif post_type == 4:
                # 纯文字
                url = "https://t.bilibili.com/{}".format(
                    raw_post["desc"]["dynamic_id_str"]
                )
            text, pic = self._get_info(post_type, card_content)
        elif post_type == 5:
            # 转发
            url = "https://t.bilibili.com/{}".format(raw_post["desc"]["dynamic_id_str"])
            text = card_content["item"]["content"]
            orig_type = card_content["item"]["orig_type"]
            orig = json.loads(card_content["origin"])
            orig_text, _ = self._get_info(self._do_get_category(orig_type), orig)
            text += "\n--------------\n"
            text += orig_text
            pic = []
        else:
            raise CategoryNotSupport(post_type)
        return Post("bilibili", text=text, url=url, pics=pic, target_name=target_name)


class Bilibililive(StatusChange):
    # Author : Sichongzou
    # Date : 2022-5-18 8:54
    # Description : bilibili开播提醒
    # E-mail : 1557157806@qq.com
    categories = {1: "开播提醒", 2: "标题更新提醒"}
    platform_name = "bilibili-live"
    enable_tag = False
    enabled = True
    is_common = True
    scheduler = BilibiliSchedConf
    name = "Bilibili直播"
    has_target = True

    @dataclass
    class Info:
        uname: str
        live_status: int
        room_id: str
        title: str
        cover_from_user: str
        keyframe: str
        category: Category = field(default=Category(0))

        def __init__(self, raw_info: dict):
            self.__dict__.update(raw_info)

        def is_live_turn_on(self, old_info: Self) -> bool:
            # 使用 & 判断直播开始
            # live_status:
            # 0:关播
            # 1:直播中
            # 2:轮播中
            if self.live_status == 1 and old_info.live_status != self.live_status:
                return True

            return False

        def is_title_update(self, old_info: Self) -> bool:
            # 使用 ^ 判断直播时标题改变
            if self.live_status == 1 and old_info.title != self.title:
                return True

            return False

    @classmethod
    async def get_target_name(
        cls, client: AsyncClient, target: Target
    ) -> Optional[str]:
        res = await client.get(
            "https://api.bilibili.com/x/space/acc/info", params={"mid": target}
        )
        res_data = json.loads(res.text)
        if res_data["code"]:
            return None
        return res_data["data"]["name"]

    async def get_status(self, target: Target) -> Info:
        params = {"uids[]": target}
        # from https://github.com/SocialSisterYi/bilibili-API-collect/blob/master/live/info.md#%E6%89%B9%E9%87%8F%E6%9F%A5%E8%AF%A2%E7%9B%B4%E6%92%AD%E9%97%B4%E7%8A%B6%E6%80%81
        res = await self.client.get(
            "http://api.live.bilibili.com/room/v1/Room/get_status_info_by_uids",
            params=params,
            timeout=4.0,
        )
        res_dict = json.loads(res.text)
        if res_dict["code"] == 0:
            data = res_dict["data"][target]

            info = self.Info(data)

            return info
        else:
            raise self.FetchError()

    def compare_status(
        self, target: Target, old_status: Info, new_status: Info
    ) -> list[RawPost]:
        if new_status.is_live_turn_on(old_status):
            # 判断开播 运算符左右有顺序要求
            current_status = deepcopy(new_status)
            current_status.category = Category(1)
            return [current_status]
        elif new_status.is_title_update(old_status):
            # 判断直播时直播间标题变更 运算符左右有顺序要求
            current_status = deepcopy(new_status)
            current_status.category = Category(2)
            return [current_status]
        else:
            return []

    def get_category(self, status: Info) -> Category:
        assert status.category != Category(0)
        return status.category

    async def parse(self, raw_info: Info) -> Post:
        url = "https://live.bilibili.com/{}".format(raw_info.room_id)
        pic = [raw_info.keyframe]
        target_name = raw_info.uname
        title = raw_info.title
        return Post(
            self.name,
            text=title,
            url=url,
            pics=pic,
            target_name=target_name,
            compress=True,
        )


class BilibiliBangumi(StatusChange):

    categories = {}
    platform_name = "bilibili-bangumi"
    enable_tag = False
    enabled = True
    is_common = True
    scheduler = BilibiliSchedConf
    name = "Bilibili剧集"
    has_target = True
    parse_target_promot = "请输入剧集主页"

    _url = "https://api.bilibili.com/pgc/review/user"

    @classmethod
    async def get_target_name(
        cls, client: AsyncClient, target: Target
    ) -> Optional[str]:
        res = await client.get(cls._url, params={"media_id": target})
        res_data = res.json()
        if res_data["code"]:
            return None
        return res_data["result"]["media"]["title"]

    @classmethod
    async def parse_target(cls, target_string: str) -> Target:
        if re.match(r"\d+", target_string):
            return Target(target_string)
        elif m := re.match(r"md(\d+)", target_string):
            return Target(m.group(1))
        elif m := re.match(
            r"(?:https?://)?www\.bilibili\.com/bangumi/media/md(\d+)/", target_string
        ):
            return Target(m.group(1))
        raise cls.ParseTargetException()

    async def get_status(self, target: Target):
        res = await self.client.get(
            self._url,
            params={"media_id": target},
            timeout=4.0,
        )
        res_dict = res.json()
        if res_dict["code"] == 0:
            return {
                "index": res_dict["result"]["media"]["new_ep"]["index"],
                "index_show": res_dict["result"]["media"]["new_ep"]["index"],
                "season_id": res_dict["result"]["media"]["season_id"],
            }
        else:
            raise self.FetchError

    def compare_status(self, target: Target, old_status, new_status) -> list[RawPost]:
        if new_status["index"] != old_status["index"]:
            return [new_status]
        else:
            return []

    async def parse(self, raw_post: RawPost) -> Post:
        detail_res = await self.client.get(
            f'http://api.bilibili.com/pgc/view/web/season?season_id={raw_post["season_id"]}'
        )
        detail_dict = detail_res.json()
        lastest_episode = None
        for episode in detail_dict["result"]["episodes"][::-1]:
            if episode["badge"] in ("", "会员"):
                lastest_episode = episode
                break
        if not lastest_episode:
            lastest_episode = detail_dict["result"]["episodes"]

        url = lastest_episode["link"]
        pic = [lastest_episode["cover"]]
        target_name = detail_dict["result"]["season_title"]
        text = lastest_episode["share_copy"]
        return Post(
            self.name,
            text=text,
            url=url,
            pics=pic,
            target_name=target_name,
            compress=True,
        )



import logging

from apscheduler.schedulers.asyncio import AsyncIOScheduler
from nonebot.log import LoguruHandler

from ..plugin_config import plugin_config
from ..send import do_send_msgs

aps = AsyncIOScheduler(timezone="Asia/Shanghai")


class CustomLogHandler(LoguruHandler):
    def filter(self, record: logging.LogRecord):
        return record.msg != (
            'Execution of job "%s" '
            "skipped: maximum number of running instances reached (%d)"
        )


if plugin_config.bison_use_queue:
    aps.add_job(do_send_msgs, "interval", seconds=0.3, coalesce=True)

    aps_logger = logging.getLogger("apscheduler")
    aps_logger.setLevel(30)
    aps_logger.handlers.clear()
    aps_logger.addHandler(CustomLogHandler())


def start_scheduler():
    aps.configure({"apscheduler.timezone": "Asia/Shanghai"})
    aps.start()



import json
import re
from collections.abc import Callable
from datetime import datetime
from typing import Any, Optional

from bs4 import BeautifulSoup as bs
from httpx import AsyncClient
from nonebot.log import logger

from ..post import Post
from ..types import *
from ..utils import SchedulerConfig, http_client
from .platform import NewMessage


class WeiboSchedConf(SchedulerConfig):
    name = "weibo.com"
    schedule_type = "interval"
    schedule_setting = {"seconds": 3}


class Weibo(NewMessage):

    categories = {
        1: "转发",
        2: "视频",
        3: "图文",
        4: "文字",
    }
    enable_tag = True
    platform_name = "weibo"
    name = "新浪微博"
    enabled = True
    is_common = True
    scheduler = WeiboSchedConf
    has_target = True
    parse_target_promot = "请输入用户主页（包含数字UID）的链接"

    @classmethod
    async def get_target_name(
        cls, client: AsyncClient, target: Target
    ) -> Optional[str]:
        param = {"containerid": "100505" + target}
        res = await client.get(
            "https://m.weibo.cn/api/container/getIndex", params=param
        )
        res_dict = json.loads(res.text)
        if res_dict.get("ok") == 1:
            return res_dict["data"]["userInfo"]["screen_name"]
        else:
            return None

    @classmethod
    async def parse_target(cls, target_text: str) -> Target:
        if re.match(r"\d+", target_text):
            return Target(target_text)
        elif match := re.match(r"(?:https?://)?weibo\.com/u/(\d+)", target_text):
            # 都2202年了应该不会有http了吧，不过还是防一手
            return Target(match.group(1))
        else:
            raise cls.ParseTargetException()

    async def get_sub_list(self, target: Target) -> list[RawPost]:
        params = {"containerid": "107603" + target}
        res = await self.client.get(
            "https://m.weibo.cn/api/container/getIndex?", params=params, timeout=4.0
        )
        res_data = json.loads(res.text)
        if not res_data["ok"]:
            return []
        custom_filter: Callable[[RawPost], bool] = lambda d: d["card_type"] == 9
        return list(filter(custom_filter, res_data["data"]["cards"]))

    def get_id(self, post: RawPost) -> Any:
        return post["mblog"]["id"]

    def filter_platform_custom(self, raw_post: RawPost) -> bool:
        return raw_post["card_type"] == 9

    def get_date(self, raw_post: RawPost) -> float:
        created_time = datetime.strptime(
            raw_post["mblog"]["created_at"], "%a %b %d %H:%M:%S %z %Y"
        )
        return created_time.timestamp()

    def get_tags(self, raw_post: RawPost) -> Optional[list[Tag]]:
        "Return Tag list of given RawPost"
        text = raw_post["mblog"]["text"]
        soup = bs(text, "html.parser")
        res = list(
            map(
                lambda x: x[1:-1],
                filter(
                    lambda s: s[0] == "#" and s[-1] == "#",
                    map(lambda x: x.text, soup.find_all("span", class_="surl-text")),
                ),
            )
        )
        super_topic_img = soup.find(
            "img", src=re.compile(r"timeline_card_small_super_default")
        )
        if super_topic_img:
            try:
                res.append(
                    super_topic_img.parent.parent.find("span", class_="surl-text").text  # type: ignore
                    + "超话"
                )
            except:
                logger.info("super_topic extract error: {}".format(text))
        return res

    def get_category(self, raw_post: RawPost) -> Category:
        if raw_post["mblog"].get("retweeted_status"):
            return Category(1)
        elif (
            raw_post["mblog"].get("page_info")
            and raw_post["mblog"]["page_info"].get("type") == "video"
        ):
            return Category(2)
        elif raw_post["mblog"].get("pics"):
            return Category(3)
        else:
            return Category(4)

    def _get_text(self, raw_text: str) -> str:
        text = raw_text.replace("<br />", "\n")
        return bs(text, "html.parser").text

    async def parse(self, raw_post: RawPost) -> Post:
        header = {
            "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
            "accept-language": "zh-CN,zh;q=0.9",
            "authority": "m.weibo.cn",
            "cache-control": "max-age=0",
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "same-origin",
            "sec-fetch-site": "same-origin",
            "upgrade-insecure-requests": "1",
            "user-agent": "Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) "
            "AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.72 "
            "Mobile Safari/537.36",
        }
        info = raw_post["mblog"]
        retweeted = False
        if info.get("retweeted_status"):
            retweeted = True
        pic_num = info["retweeted_status"]["pic_num"] if retweeted else info["pic_num"]
        if info["isLongText"] or pic_num > 9:
            res = await self.client.get(
                "https://m.weibo.cn/detail/{}".format(info["mid"]), headers=header
            )
            try:
                match = re.search(r'"status": ([\s\S]+),\s+"call"', res.text)
                assert match
                full_json_text = match.group(1)
                info = json.loads(full_json_text)
            except:
                logger.info(
                    "detail message error: https://m.weibo.cn/detail/{}".format(
                        info["mid"]
                    )
                )
        parsed_text = self._get_text(info["text"])
        raw_pics_list = (
            info["retweeted_status"].get("pics", [])
            if retweeted
            else info.get("pics", [])
        )
        pic_urls = [img["large"]["url"] for img in raw_pics_list]
        pics = []
        for pic_url in pic_urls:
            async with http_client(headers={"referer": "https://weibo.com"}) as client:
                res = await client.get(pic_url)
                res.raise_for_status()
                pics.append(res.content)
        detail_url = "https://weibo.com/{}/{}".format(info["user"]["id"], info["bid"])
        # return parsed_text, detail_url, pic_urls
        return Post(
            "weibo",
            text=parsed_text,
            url=detail_url,
            pics=pics,
            target_name=info["user"]["screen_name"],
        )



import asyncio
from datetime import datetime
from typing import Optional, Type, cast

from nonebot import on_command
from nonebot.adapters.onebot.v11 import Bot, Event, MessageEvent
from nonebot.adapters.onebot.v11.event import GroupMessageEvent, PrivateMessageEvent
from nonebot.adapters.onebot.v11.message import Message
from nonebot.adapters.onebot.v11.permission import GROUP_ADMIN, GROUP_OWNER
from nonebot.internal.params import ArgStr
from nonebot.internal.rule import Rule
from nonebot.matcher import Matcher
from nonebot.params import Depends, EventPlainText, EventToMe
from nonebot.permission import SUPERUSER
from nonebot.rule import to_me
from nonebot.typing import T_State

from .apis import check_sub_target
from .config import config
from .config.db_config import SubscribeDupException
from .platform import Platform, platform_manager
from .plugin_config import plugin_config
from .types import Category, Target, User
from .utils import parse_text


def _gen_prompt_template(prompt: str):
    if hasattr(Message, "template"):
        return Message.template(prompt)
    return prompt


def _configurable_to_me(to_me: bool = EventToMe()):
    if plugin_config.bison_to_me:
        return to_me
    else:
        return True


configurable_to_me = Rule(_configurable_to_me)


common_platform = [
    p.platform_name
    for p in filter(
        lambda platform: platform.enabled and platform.is_common,
        platform_manager.values(),
    )
]


def ensure_user_info(matcher: Type[Matcher]):
    async def _check_user_info(state: T_State):
        if not state.get("target_user_info"):
            await matcher.finish(
                "No target_user_info set, this shouldn't happen, please issue"
            )

    return _check_user_info


async def set_target_user_info(event: MessageEvent, state: T_State):
    if isinstance(event, GroupMessageEvent):
        user = User(event.group_id, "group")
        state["target_user_info"] = user
    elif isinstance(event, PrivateMessageEvent):
        user = User(event.user_id, "private")
        state["target_user_info"] = user


def do_add_sub(add_sub: Type[Matcher]):
    add_sub.handle()(ensure_user_info(add_sub))

    @add_sub.handle()
    async def init_promote(state: T_State):
        state["_prompt"] = (
            "请输入想要订阅的平台，目前支持，请输入冒号左边的名称：\n"
            + "".join(
                [
                    "{}：{}\n".format(
                        platform_name, platform_manager[platform_name].name
                    )
                    for platform_name in common_platform
                ]
            )
            + "要查看全部平台请输入：“全部”\n中止订阅过程请输入：“取消”"
        )

    async def parse_platform(event: MessageEvent, state: T_State) -> None:
        if not isinstance(state["platform"], Message):
            return
        platform = str(event.get_message()).strip()
        if platform == "全部":
            message = "全部平台\n" + "\n".join(
                [
                    "{}：{}".format(platform_name, platform.name)
                    for platform_name, platform in platform_manager.items()
                ]
            )
            await add_sub.reject(message)
        elif platform == "取消":
            await add_sub.finish("已中止订阅")
        elif platform in platform_manager:
            state["platform"] = platform
        else:
            await add_sub.reject("平台输入错误")

    @add_sub.got(
        "platform", _gen_prompt_template("{_prompt}"), [Depends(parse_platform)]
    )
    async def init_id(state: T_State):
        cur_platform = platform_manager[state["platform"]]
        if cur_platform.has_target:
            state["_prompt"] = (
                ("1." + cur_platform.parse_target_promot + "\n2.")
                if cur_platform.parse_target_promot
                else ""
            ) + "请输入订阅用户的id\n查询id获取方法请回复:“查询”"
        else:
            state["id"] = "default"
            state["name"] = await check_sub_target(state["platform"], Target(""))

    async def parse_id(event: MessageEvent, state: T_State):
        if not isinstance(state["id"], Message):
            return
        target = str(event.get_message()).strip()
        try:
            if target == "查询":
                raise LookupError
            if target == "取消":
                raise KeyboardInterrupt
            platform = platform_manager[state["platform"]]
            target = await platform.parse_target(target)
            name = await check_sub_target(state["platform"], target)
            if not name:
                raise ValueError
            state["id"] = target
            state["name"] = name
        except (LookupError):
            url = "https://nonebot-bison.netlify.app/usage/#%E6%89%80%E6%94%AF%E6%8C%81%E5%B9%B3%E5%8F%B0%E7%9A%84-uid"
            title = "Bison所支持的平台UID"
            content = "查询相关平台的uid格式或获取方式"
            image = "https://s3.bmp.ovh/imgs/2022/03/ab3cc45d83bd3dd3.jpg"
            getId_share = f"[CQ:share,url={url},title={title},content={content},image={image}]"  # 缩短字符串格式长度，以及方便后续修改为消息段格式
            await add_sub.reject(Message(getId_share))
        except (KeyboardInterrupt):
            await add_sub.finish("已中止订阅")
        except (ValueError):
            await add_sub.reject("id输入错误")
        except (Platform.ParseTargetException):
            await add_sub.reject("不能从你的输入中提取出id，请检查你输入的内容是否符合预期")
        else:
            await add_sub.send(
                "即将订阅的用户为:{} {} {}\n如有错误请输入“取消”重新订阅".format(
                    state["platform"], state["name"], state["id"]
                )
            )

    @add_sub.got("id", _gen_prompt_template("{_prompt}"), [Depends(parse_id)])
    async def init_cat(state: T_State):
        if not platform_manager[state["platform"]].categories:
            state["cats"] = []
            return
        state["_prompt"] = "请输入要订阅的类别，以空格分隔，支持的类别有：{}".format(
            " ".join(list(platform_manager[state["platform"]].categories.values()))
        )

    async def parser_cats(event: MessageEvent, state: T_State):
        if not isinstance(state["cats"], Message):
            return
        res = []
        for cat in str(event.get_message()).strip().split():
            if cat == "取消":
                await add_sub.finish("已中止订阅")
            elif cat not in platform_manager[state["platform"]].reverse_category:
                await add_sub.reject("不支持 {}".format(cat))
            res.append(platform_manager[state["platform"]].reverse_category[cat])
        state["cats"] = res

    @add_sub.got("cats", _gen_prompt_template("{_prompt}"), [Depends(parser_cats)])
    async def init_tag(state: T_State):
        if not platform_manager[state["platform"]].enable_tag:
            state["tags"] = []
            return
        state["_prompt"] = '请输入要订阅/屏蔽的tag(不含#号)\n多个tag请使用空格隔开\n具体规则回复"详情"'

    async def parser_tags(event: MessageEvent, state: T_State):
        if not isinstance(state["tags"], Message):
            return
        if str(event.get_message()).strip() == "取消":  # 一般不会有叫 取消 的tag吧
            await add_sub.finish("已中止订阅")
        if str(event.get_message()).strip() == "详情":
            await add_sub.reject(
                '订阅tag直接输入tag内容\n订阅所有tag输入"全部标签"\n屏蔽tag请在tag名称前添加~号\n详见https://nonebot-bison.netlify.app/usage/#平台订阅标签-tag'
            )
        if str(event.get_message()).strip() == "全部标签":
            state["tags"] = []
        else:
            state["tags"] = str(event.get_message()).strip().split()

    @add_sub.got("tags", _gen_prompt_template("{_prompt}"), [Depends(parser_tags)])
    async def add_sub_process(event: Event, state: T_State):
        user = cast(User, state.get("target_user_info"))
        assert isinstance(user, User)
        try:
            await config.add_subscribe(
                # state.get("_user_id") or event.group_id,
                # user_type="group",
                user=user.user,
                user_type=user.user_type,
                target=state["id"],
                target_name=state["name"],
                platform_name=state["platform"],
                cats=state.get("cats", []),
                tags=state.get("tags", []),
            )
        except SubscribeDupException:
            await add_sub.finish(f"添加 {state['name']} 失败: 已存在该订阅")
        except Exception as e:
            await add_sub.finish(f"添加 {state['name']} 失败: {e}")
        await add_sub.finish("添加 {} 成功".format(state["name"]))


def do_query_sub(query_sub: Type[Matcher]):
    query_sub.handle()(ensure_user_info(query_sub))

    @query_sub.handle()
    async def _(state: T_State):
        user_info = state["target_user_info"]
        assert isinstance(user_info, User)
        sub_list = await config.list_subscribe(
            # state.get("_user_id") or event.group_id, "group"
            user_info.user,
            user_info.user_type,
        )
        res = "订阅的帐号为：\n"
        for sub in sub_list:
            res += "{} {} {}".format(
                # sub["target_type"], sub["target_name"], sub["target"]
                sub.target.platform_name,
                sub.target.target_name,
                sub.target.target,
            )
            platform = platform_manager[sub.target.platform_name]
            if platform.categories:
                res += " [{}]".format(
                    ", ".join(
                        map(lambda x: platform.categories[Category(x)], sub.categories)
                    )
                )
            if platform.enable_tag:
                res += " {}".format(", ".join(sub.tags))
            res += "\n"
        await query_sub.finish(Message(await parse_text(res)))


def do_del_sub(del_sub: Type[Matcher]):
    del_sub.handle()(ensure_user_info(del_sub))

    @del_sub.handle()
    async def send_list(bot: Bot, event: Event, state: T_State):
        user_info = state["target_user_info"]
        assert isinstance(user_info, User)
        try:
            sub_list = await config.list_subscribe(
                # state.get("_user_id") or event.group_id, "group"
                user_info.user,
                user_info.user_type,
            )
            assert sub_list
        except AssertionError:
            await del_sub.finish("暂无已订阅账号\n请使用“添加订阅”命令添加订阅")
        else:
            res = "订阅的帐号为：\n"
            state["sub_table"] = {}
            for index, sub in enumerate(sub_list, 1):
                state["sub_table"][index] = {
                    "platform_name": sub.target.platform_name,
                    "target": sub.target.target,
                }
                res += "{} {} {} {}\n".format(
                    index,
                    sub.target.platform_name,
                    sub.target.target_name,
                    sub.target.target,
                )
                platform = platform_manager[sub.target.platform_name]
                if platform.categories:
                    res += " [{}]".format(
                        ", ".join(
                            map(
                                lambda x: platform.categories[Category(x)],
                                sub.categories,
                            )
                        )
                    )
                if platform.enable_tag:
                    res += " {}".format(", ".join(sub.tags))
                res += "\n"
            res += "请输入要删除的订阅的序号\n输入'取消'中止"
            await bot.send(event=event, message=Message(await parse_text(res)))

    @del_sub.receive()
    async def do_del(event: Event, state: T_State):
        user_msg = str(event.get_message()).strip()
        if user_msg == "取消":
            await del_sub.finish("删除中止")
        try:
            index = int(user_msg)
            user_info = state["target_user_info"]
            assert isinstance(user_info, User)
            await config.del_subscribe(
                # state.get("_user_id") or event.group_id,
                # "group",
                user_info.user,
                user_info.user_type,
                **state["sub_table"][index],
            )
        except Exception as e:
            await del_sub.reject("删除错误")
        else:
            await del_sub.finish("删除成功")


add_sub_matcher = on_command(
    "添加订阅",
    rule=configurable_to_me,
    permission=GROUP_ADMIN | GROUP_OWNER | SUPERUSER,
    priority=5,
    block=True,
)
add_sub_matcher.handle()(set_target_user_info)
do_add_sub(add_sub_matcher)


query_sub_matcher = on_command("查询订阅", rule=configurable_to_me, priority=5, block=True)
query_sub_matcher.handle()(set_target_user_info)
do_query_sub(query_sub_matcher)


del_sub_matcher = on_command(
    "删除订阅",
    rule=configurable_to_me,
    permission=GROUP_ADMIN | GROUP_OWNER | SUPERUSER,
    priority=5,
    block=True,
)
del_sub_matcher.handle()(set_target_user_info)
do_del_sub(del_sub_matcher)

group_manage_matcher = on_command(
    "群管理", rule=to_me(), permission=SUPERUSER, priority=4, block=True
)


@group_manage_matcher.handle()
async def send_group_list_private(bot: Bot, event: GroupMessageEvent, state: T_State):
    await group_manage_matcher.finish(Message("该功能只支持私聊使用，请私聊Bot"))


@group_manage_matcher.handle()
async def send_group_list(bot: Bot, event: PrivateMessageEvent, state: T_State):
    groups = await bot.call_api("get_group_list")
    res_text = "请选择需要管理的群：\n"
    group_number_idx = {}
    for idx, group in enumerate(groups, 1):
        group_number_idx[idx] = group["group_id"]
        res_text += f'{idx}. {group["group_id"]} - {group["group_name"]}\n'
    res_text += "请输入左侧序号\n中止操作请输入'取消'"
    # await group_manage_matcher.send(res_text)
    state["_prompt"] = res_text
    state["group_number_idx"] = group_number_idx


async def _parse_group_idx(state: T_State, event_msg: str = EventPlainText()):
    if not isinstance(state["group_idx"], Message):
        return
    group_number_idx: Optional[dict[int, int]] = state.get("group_number_idx")
    assert group_number_idx
    try:
        assert event_msg != "取消", "userAbort"
        idx = int(event_msg)
        assert idx in group_number_idx.keys(), "idxNotInList"
        state["group_idx"] = idx
    except AssertionError as AE:
        errType = AE.args[0]
        if errType == "userAbort":
            await group_manage_matcher.finish("已取消")
        elif errType == "idxNotInList":
            await group_manage_matcher.reject("请输入正确序号")


@group_manage_matcher.got(
    "group_idx", _gen_prompt_template("{_prompt}"), [Depends(_parse_group_idx)]
)
async def do_choose_group_number(state: T_State):
    group_number_idx: dict[int, int] = state["group_number_idx"]
    idx: int = state["group_idx"]
    group_id = group_number_idx[idx]
    state["target_user_info"] = User(user=group_id, user_type="group")


async def _check_command(event_msg: str = EventPlainText()):
    if event_msg not in {"添加订阅", "查询订阅", "删除订阅", "取消"}:
        await group_manage_matcher.reject("请输入正确的命令")
    return


@group_manage_matcher.got(
    "command", "请输入需要使用的命令：添加订阅，查询订阅，删除订阅，取消", [Depends(_check_command)]
)
async def do_dispatch_command(
    bot: Bot,
    event: MessageEvent,
    state: T_State,
    matcher: Matcher,
    command: str = ArgStr(),
):
    if command == "取消":
        await group_manage_matcher.finish("已取消")
    permission = await matcher.update_permission(bot, event)
    new_matcher = Matcher.new(
        "message",
        Rule(),
        permission,
        handlers=None,
        temp=True,
        priority=0,
        block=True,
        plugin=matcher.plugin,
        module=matcher.module,
        expire_time=datetime.now(),
        default_state=matcher.state,
        default_type_updater=matcher.__class__._default_type_updater,
        default_permission_updater=matcher.__class__._default_permission_updater,
    )
    if command == "查询订阅":
        do_query_sub(new_matcher)
    elif command == "添加订阅":
        do_add_sub(new_matcher)
    else:
        do_del_sub(new_matcher)
    new_matcher_ins = new_matcher()
    asyncio.create_task(new_matcher_ins.run(bot, event, state))



import json
import os
from collections import defaultdict
from datetime import datetime
from os import path
from pathlib import Path
from typing import DefaultDict, Literal, Mapping, TypedDict

import nonebot
from nonebot.log import logger
from tinydb import Query, TinyDB

from ..platform import platform_manager
from ..plugin_config import plugin_config
from ..types import Target, User
from ..utils import Singleton
from .utils import NoSuchSubscribeException, NoSuchUserException

supported_target_type = platform_manager.keys()


def get_config_path() -> tuple[str, str]:
    if plugin_config.bison_config_path:
        data_dir = plugin_config.bison_config_path
    else:
        working_dir = os.getcwd()
        data_dir = path.join(working_dir, "data")
    if not path.isdir(data_dir):
        os.makedirs(data_dir)
    old_path = path.join(data_dir, "hk_reporter.json")
    new_path = path.join(data_dir, "bison.json")
    deprecated_maker_path = path.join(data_dir, "bison.json.deprecated")
    if os.path.exists(old_path) and not os.path.exists(new_path):
        os.rename(old_path, new_path)
    return new_path, deprecated_maker_path


def drop():
    if plugin_config.bison_config_path:
        data_dir = plugin_config.bison_config_path
    else:
        working_dir = os.getcwd()
        data_dir = path.join(working_dir, "data")
    old_path = path.join(data_dir, "bison.json")
    deprecated_marker_path = path.join(data_dir, "bison.json.deprecated")
    if os.path.exists(old_path):
        config.db.close()
        config.available = False
        with open(deprecated_marker_path, "w") as file:
            content = {
                "migration_time": datetime.now().isoformat(),
            }
            file.write(json.dumps(content))
        return True
    return False


class SubscribeContent(TypedDict):
    target: str
    target_type: str
    target_name: str
    cats: list[int]
    tags: list[str]


class ConfigContent(TypedDict):
    user: int
    user_type: Literal["group", "private"]
    subs: list[SubscribeContent]


class Config(metaclass=Singleton):
    "Dropping it!"

    migrate_version = 2

    def __init__(self):
        self._do_init()

    def _do_init(self):
        path, deprecated_marker_path = get_config_path()
        if Path(deprecated_marker_path).exists():
            self.available = False
        elif Path(path).exists():
            self.available = True
            self.db = TinyDB(path, encoding="utf-8")
            self.kv_config = self.db.table("kv")
            self.user_target = self.db.table("user_target")
            self.target_user_cache: dict[str, defaultdict[Target, list[User]]] = {}
            self.target_user_cat_cache = {}
            self.target_user_tag_cache = {}
            self.target_list = {}
            self.next_index: DefaultDict[str, int] = defaultdict(lambda: 0)
        else:
            self.available = False

    def add_subscribe(
        self, user, user_type, target, target_name, target_type, cats, tags
    ):
        user_query = Query()
        query = (user_query.user == user) & (user_query.user_type == user_type)
        if user_data := self.user_target.get(query):
            # update
            subs: list = user_data.get("subs", [])
            subs.append(
                {
                    "target": target,
                    "target_type": target_type,
                    "target_name": target_name,
                    "cats": cats,
                    "tags": tags,
                }
            )
            self.user_target.update({"subs": subs}, query)
        else:
            # insert
            self.user_target.insert(
                {
                    "user": user,
                    "user_type": user_type,
                    "subs": [
                        {
                            "target": target,
                            "target_type": target_type,
                            "target_name": target_name,
                            "cats": cats,
                            "tags": tags,
                        }
                    ],
                }
            )
        self.update_send_cache()

    def list_subscribe(self, user, user_type) -> list[SubscribeContent]:
        query = Query()
        if user_sub := self.user_target.get(
            (query.user == user) & (query.user_type == user_type)
        ):
            return user_sub["subs"]
        return []

    def get_all_subscribe(self):
        return self.user_target

    def del_subscribe(self, user, user_type, target, target_type):
        user_query = Query()
        query = (user_query.user == user) & (user_query.user_type == user_type)
        if not (query_res := self.user_target.get(query)):
            raise NoSuchUserException()
        subs = query_res.get("subs", [])
        for idx, sub in enumerate(subs):
            if sub.get("target") == target and sub.get("target_type") == target_type:
                subs.pop(idx)
                self.user_target.update({"subs": subs}, query)
                self.update_send_cache()
                return
        raise NoSuchSubscribeException()

    def update_subscribe(
        self, user, user_type, target, target_name, target_type, cats, tags
    ):
        user_query = Query()
        query = (user_query.user == user) & (user_query.user_type == user_type)
        if user_data := self.user_target.get(query):
            # update
            subs: list = user_data.get("subs", [])
            find_flag = False
            for item in subs:
                if item["target"] == target and item["target_type"] == target_type:
                    item["target_name"], item["cats"], item["tags"] = (
                        target_name,
                        cats,
                        tags,
                    )
                    find_flag = True
                    break
            if not find_flag:
                raise NoSuchSubscribeException()
            self.user_target.update({"subs": subs}, query)
        else:
            raise NoSuchUserException()
        self.update_send_cache()

    def update_send_cache(self):
        res = {target_type: defaultdict(list) for target_type in supported_target_type}
        cat_res = {
            target_type: defaultdict(lambda: defaultdict(list))
            for target_type in supported_target_type
        }
        tag_res = {
            target_type: defaultdict(lambda: defaultdict(list))
            for target_type in supported_target_type
        }
        # res = {target_type: defaultdict(lambda: defaultdict(list)) for target_type in supported_target_type}
        to_del = []
        for user in self.user_target.all():
            for sub in user.get("subs", []):
                if not sub.get("target_type") in supported_target_type:
                    to_del.append(
                        {
                            "user": user["user"],
                            "user_type": user["user_type"],
                            "target": sub["target"],
                            "target_type": sub["target_type"],
                        }
                    )
                    continue
                res[sub["target_type"]][sub["target"]].append(
                    User(user["user"], user["user_type"])
                )
                cat_res[sub["target_type"]][sub["target"]][
                    "{}-{}".format(user["user_type"], user["user"])
                ] = sub["cats"]
                tag_res[sub["target_type"]][sub["target"]][
                    "{}-{}".format(user["user_type"], user["user"])
                ] = sub["tags"]
        self.target_user_cache = res
        self.target_user_cat_cache = cat_res
        self.target_user_tag_cache = tag_res
        for target_type in self.target_user_cache:
            self.target_list[target_type] = list(
                self.target_user_cache[target_type].keys()
            )

        logger.info(f"Deleting {to_del}")
        for d in to_del:
            self.del_subscribe(**d)

    def get_sub_category(self, target_type, target, user_type, user):
        return self.target_user_cat_cache[target_type][target][
            "{}-{}".format(user_type, user)
        ]

    def get_sub_tags(self, target_type, target, user_type, user):
        return self.target_user_tag_cache[target_type][target][
            "{}-{}".format(user_type, user)
        ]

    def get_next_target(self, target_type):
        # FIXME 插入或删除target后对队列的影响（但是并不是大问题
        if not self.target_list[target_type]:
            return None
        self.next_index[target_type] %= len(self.target_list[target_type])
        res = self.target_list[target_type][self.next_index[target_type]]
        self.next_index[target_type] += 1
        return res


def start_up():
    config = Config()
    if not config.available:
        return
    if not (search_res := config.kv_config.search(Query().name == "version")):
        config.kv_config.insert({"name": "version", "value": config.migrate_version})
    elif search_res[0].get("value") < config.migrate_version:
        query = Query()
        version_query = query.name == "version"
        cur_version = search_res[0].get("value")
        if cur_version == 1:
            cur_version = 2
            for user_conf in config.user_target.all():
                conf_id = user_conf.doc_id
                subs = user_conf["subs"]
                for sub in subs:
                    sub["cats"] = []
                    sub["tags"] = []
                config.user_target.update({"subs": subs}, doc_ids=[conf_id])
        config.kv_config.update({"value": config.migrate_version}, version_query)
        # do migration
    config.update_send_cache()


config = Config()



"""alter type

Revision ID: 4a46ba54a3f3
Revises: c97c445e2bdb
Create Date: 2022-03-27 21:50:10.911649

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "4a46ba54a3f3"
down_revision = "c97c445e2bdb"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("subscribe", schema=None) as batch_op:
        batch_op.alter_column(
            "categories",
            existing_type=sa.VARCHAR(length=1024),
            type_=sa.JSON(),
            existing_nullable=True,
        )
        batch_op.alter_column(
            "tags",
            existing_type=sa.VARCHAR(length=1024),
            type_=sa.JSON(),
            existing_nullable=True,
        )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("subscribe", schema=None) as batch_op:
        batch_op.alter_column(
            "tags",
            existing_type=sa.JSON(),
            type_=sa.VARCHAR(length=1024),
            existing_nullable=True,
        )
        batch_op.alter_column(
            "categories",
            existing_type=sa.JSON(),
            type_=sa.VARCHAR(length=1024),
            existing_nullable=True,
        )

    # ### end Alembic commands ###



from dataclasses import dataclass, field
from functools import reduce
from io import BytesIO
from typing import Optional, Union

from nonebot.adapters.onebot.v11.message import Message, MessageSegment
from nonebot.log import logger
from PIL import Image

from ..utils import http_client, parse_text
from .abstract_post import AbstractPost, BasePost, OptionalMixin


@dataclass
class _Post(BasePost):

    target_type: str
    text: str
    url: Optional[str] = None
    target_name: Optional[str] = None
    pics: list[Union[str, bytes]] = field(default_factory=list)

    _message: Optional[list[MessageSegment]] = None
    _pic_message: Optional[list[MessageSegment]] = None

    async def _pic_url_to_image(self, data: Union[str, bytes]) -> Image.Image:
        pic_buffer = BytesIO()
        if isinstance(data, str):
            async with http_client() as client:
                res = await client.get(data)
            pic_buffer.write(res.content)
        else:
            pic_buffer.write(data)
        return Image.open(pic_buffer)

    def _check_image_square(self, size: tuple[int, int]) -> bool:
        return abs(size[0] - size[1]) / size[0] < 0.05

    async def _pic_merge(self) -> None:
        if len(self.pics) < 3:
            return
        first_image = await self._pic_url_to_image(self.pics[0])
        if not self._check_image_square(first_image.size):
            return
        images: list[Image.Image] = [first_image]
        # first row
        for i in range(1, 3):
            cur_img = await self._pic_url_to_image(self.pics[i])
            if not self._check_image_square(cur_img.size):
                return
            if cur_img.size[1] != images[0].size[1]:  # height not equal
                return
            images.append(cur_img)
        _tmp = 0
        x_coord = [0]
        for i in range(3):
            _tmp += images[i].size[0]
            x_coord.append(_tmp)
        y_coord = [0, first_image.size[1]]

        async def process_row(row: int) -> bool:
            if len(self.pics) < (row + 1) * 3:
                return False
            row_first_img = await self._pic_url_to_image(self.pics[row * 3])
            if not self._check_image_square(row_first_img.size):
                return False
            if row_first_img.size[0] != images[0].size[0]:
                return False
            image_row: list[Image.Image] = [row_first_img]
            for i in range(row * 3 + 1, row * 3 + 3):
                cur_img = await self._pic_url_to_image(self.pics[i])
                if not self._check_image_square(cur_img.size):
                    return False
                if cur_img.size[1] != row_first_img.size[1]:
                    return False
                if cur_img.size[0] != images[i % 3].size[0]:
                    return False
                image_row.append(cur_img)
            images.extend(image_row)
            y_coord.append(y_coord[-1] + row_first_img.size[1])
            return True

        if await process_row(1):
            matrix = (3, 2)
        else:
            matrix = (3, 1)
        if await process_row(2):
            matrix = (3, 3)
        logger.info("trigger merge image")
        target = Image.new("RGB", (x_coord[-1], y_coord[-1]))
        for y in range(matrix[1]):
            for x in range(matrix[0]):
                target.paste(
                    images[y * matrix[0] + x],
                    (x_coord[x], y_coord[y], x_coord[x + 1], y_coord[y + 1]),
                )
        target_io = BytesIO()
        target.save(target_io, "JPEG")
        self.pics = self.pics[matrix[0] * matrix[1] :]
        self.pics.insert(0, target_io.getvalue())

    async def generate_text_messages(self) -> list[MessageSegment]:

        if self._message is None:
            await self._pic_merge()
            msg_segments: list[MessageSegment] = []
            text = ""
            if self.text:
                text += "{}".format(
                    self.text if len(self.text) < 500 else self.text[:500] + "..."
                )
            if text:
                text += "\n"
            text += "来源: {}".format(self.target_type)
            if self.target_name:
                text += " {}".format(self.target_name)
            if self.url:
                text += " \n详情: {}".format(self.url)
            msg_segments.append(MessageSegment.text(text))
            for pic in self.pics:
                msg_segments.append(MessageSegment.image(pic))
            self._message = msg_segments
        return self._message

    async def generate_pic_messages(self) -> list[MessageSegment]:

        if self._pic_message is None:
            await self._pic_merge()
            msg_segments: list[MessageSegment] = []
            text = ""
            if self.text:
                text += "{}".format(self.text)
                text += "\n"
            text += "来源: {}".format(self.target_type)
            if self.target_name:
                text += " {}".format(self.target_name)
            msg_segments.append(await parse_text(text))
            if not self.target_type == "rss" and self.url:
                msg_segments.append(MessageSegment.text(self.url))
            for pic in self.pics:
                msg_segments.append(MessageSegment.image(pic))
            self._pic_message = msg_segments
        return self._pic_message

    def __str__(self):
        return "type: {}\nfrom: {}\ntext: {}\nurl: {}\npic: {}".format(
            self.target_type,
            self.target_name,
            self.text if len(self.text) < 500 else self.text[:500] + "...",
            self.url,
            ", ".join(
                map(
                    lambda x: "b64img"
                    if isinstance(x, bytes) or x.startswith("base64")
                    else x,
                    self.pics,
                )
            ),
        )


@dataclass
class Post(AbstractPost, _Post):
    pass



from typing import Any, Optional

from httpx import AsyncClient

from ..post import Post
from ..types import RawPost, Target
from ..utils import http_client, scheduler
from .platform import NewMessage


class FF14(NewMessage):

    categories = {}
    platform_name = "ff14"
    name = "最终幻想XIV官方公告"
    enable_tag = False
    enabled = True
    is_common = False
    scheduler_class = "ff14"
    scheduler = scheduler("interval", {"seconds": 60})
    has_target = False

    @classmethod
    async def get_target_name(
        cls, client: AsyncClient, target: Target
    ) -> Optional[str]:
        return "最终幻想XIV官方公告"

    async def get_sub_list(self, _) -> list[RawPost]:
        async with http_client() as client:
            raw_data = await client.get(
                "https://ff.web.sdo.com/inc/newdata.ashx?url=List?gameCode=ff&category=5309,5310,5311,5312,5313&pageIndex=0&pageSize=5"
            )
            return raw_data.json()["Data"]

    def get_id(self, post: RawPost) -> Any:
        """用发布时间当作 ID

        因为有时候官方会直接编辑以前的文章内容
        """
        return post["PublishDate"]

    def get_date(self, _: RawPost) -> None:
        return None

    async def parse(self, raw_post: RawPost) -> Post:
        text = f'{raw_post["Title"]}\n{raw_post["Summary"]}'
        url = raw_post["Author"]
        return Post("ff14", text=text, url=url, target_name="最终幻想XIV官方公告")



from pathlib import Path

from alembic.config import Config
from alembic.runtime.environment import EnvironmentContext
from alembic.script.base import ScriptDirectory
from nonebot.log import logger
from nonebot_plugin_datastore import PluginData, db
from nonebot_plugin_datastore.db import get_engine
from sqlalchemy.engine.base import Connection
from sqlalchemy.ext.asyncio.session import AsyncSession

from .config_legacy import ConfigContent, config, drop
from .db_model import Base, Subscribe, Target, User

DATA = PluginData("bison")


async def data_migrate():
    if config.available:
        logger.warning("You are still using legacy db, migrating to sqlite")
        all_subs: list[ConfigContent] = list(
            map(lambda item: ConfigContent(**item), config.get_all_subscribe().all())
        )
        async with AsyncSession(get_engine()) as sess:
            user_to_create = []
            subscribe_to_create = []
            platform_target_map: dict[str, tuple[Target, str, int]] = {}
            for user in all_subs:
                db_user = User(uid=user["user"], type=user["user_type"])
                user_to_create.append(db_user)
                user_sub_set = set()
                for sub in user["subs"]:
                    target = sub["target"]
                    platform_name = sub["target_type"]
                    target_name = sub["target_name"]
                    key = f"{target}-{platform_name}"
                    if key in user_sub_set:
                        # a user subscribe a target twice
                        logger.error(
                            f"用户 {user['user_type']}-{user['user']} 订阅了 {platform_name}-{target_name} 两次，"
                            "随机采用了一个订阅"
                        )
                        continue
                    user_sub_set.add(key)
                    if key in platform_target_map.keys():
                        target_obj, ext_user_type, ext_user = platform_target_map[key]
                        if target_obj.target_name != target_name:
                            # GG
                            logger.error(
                                f"你的旧版本数据库中存在数据不一致问题，请完成迁移后执行重新添加{platform_name}平台的{target}"
                                f"它的名字可能为{target_obj.target_name}或{target_name}"
                            )

                    else:
                        target_obj = Target(
                            platform_name=platform_name,
                            target_name=target_name,
                            target=target,
                        )
                        platform_target_map[key] = (
                            target_obj,
                            user["user_type"],
                            user["user"],
                        )
                    subscribe_obj = Subscribe(
                        user=db_user,
                        target=target_obj,
                        categories=sub["cats"],
                        tags=sub["tags"],
                    )
                    subscribe_to_create.append(subscribe_obj)
            sess.add_all(
                user_to_create
                + list(map(lambda x: x[0], platform_target_map.values()))
                + subscribe_to_create
            )
            await sess.commit()
            drop()
            logger.info("migrate success")


async def upgrade_db():
    alembic_cfg = Config()
    alembic_cfg.set_main_option(
        "script_location", str(Path(__file__).parent.joinpath("migrate"))
    )

    script = ScriptDirectory.from_config(alembic_cfg)
    engine = db.get_engine()
    env = EnvironmentContext(alembic_cfg, script)

    def migrate_fun(revision, context):
        return script._upgrade_revs("head", revision)

    def do_run_migration(connection: Connection):
        env.configure(
            connection,
            target_metadata=Base.metadata,
            fn=migrate_fun,
            render_as_batch=True,
        )
        with env.begin_transaction():
            env.run_migrations()
        logger.info("Finish auto migrate")

    async with engine.connect() as connection:
        await connection.run_sync(do_run_migration)

    await data_migrate()



from typing import Type

from ..config import config
from ..config.db_model import Target
from ..platform import platform_manager
from ..types import Target as T_Target
from ..utils import SchedulerConfig
from .scheduler import Scheduler

scheduler_dict: dict[Type[SchedulerConfig], Scheduler] = {}


async def init_scheduler():
    _schedule_class_dict: dict[Type[SchedulerConfig], list[Target]] = {}
    _schedule_class_platform_dict: dict[Type[SchedulerConfig], list[str]] = {}
    for platform in platform_manager.values():
        scheduler_config = platform.scheduler
        if not hasattr(scheduler_config, "name") or not scheduler_config.name:
            scheduler_config.name = f"AnonymousScheduleConfig[{platform.platform_name}]"

        platform_name = platform.platform_name
        targets = await config.get_platform_target(platform_name)
        if scheduler_config not in _schedule_class_dict:
            _schedule_class_dict[scheduler_config] = targets
        else:
            _schedule_class_dict[scheduler_config].extend(targets)
        if scheduler_config not in _schedule_class_platform_dict:
            _schedule_class_platform_dict[scheduler_config] = [platform_name]
        else:
            _schedule_class_platform_dict[scheduler_config].append(platform_name)
    for scheduler_config, target_list in _schedule_class_dict.items():
        schedulable_args = []
        for target in target_list:
            schedulable_args.append((target.platform_name, T_Target(target.target)))
        platform_name_list = _schedule_class_platform_dict[scheduler_config]
        scheduler_dict[scheduler_config] = Scheduler(
            scheduler_config, schedulable_args, platform_name_list
        )


async def handle_insert_new_target(platform_name: str, target: T_Target):
    platform = platform_manager[platform_name]
    scheduler_obj = scheduler_dict[platform.scheduler]
    scheduler_obj.insert_new_schedulable(platform_name, target)


async def handle_delete_target(platform_name: str, target: T_Target):
    platform = platform_manager[platform_name]
    scheduler_obj = scheduler_dict[platform.scheduler]
    scheduler_obj.delete_schedulable(platform_name, target)


config.register_add_target_hook(handle_insert_new_target)
config.register_delete_target_hook(handle_delete_target)



"""init db

Revision ID: 0571870f5222
Revises: 
Create Date: 2022-03-21 19:18:13.762626

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "0571870f5222"
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "target",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("platform_name", sa.String(length=20), nullable=False),
        sa.Column("target", sa.String(length=1024), nullable=False),
        sa.Column("target_name", sa.String(length=1024), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "user",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("type", sa.String(length=20), nullable=False),
        sa.Column("uid", sa.Integer(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "subscribe",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("target_id", sa.Integer(), nullable=True),
        sa.Column("user_id", sa.Integer(), nullable=True),
        sa.Column("categories", sa.String(length=1024), nullable=True),
        sa.Column("tags", sa.String(length=1024), nullable=True),
        sa.ForeignKeyConstraint(
            ["target_id"],
            ["target.id"],
        ),
        sa.ForeignKeyConstraint(
            ["user_id"],
            ["user.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("subscribe")
    op.drop_table("user")
    op.drop_table("target")
    # ### end Alembic commands ###



import os
from dataclasses import dataclass
from pathlib import Path
from typing import Union

import socketio
from fastapi.staticfiles import StaticFiles
from nonebot import get_driver, on_command
from nonebot.adapters.onebot.v11 import Bot
from nonebot.adapters.onebot.v11.event import GroupMessageEvent, PrivateMessageEvent
from nonebot.drivers.fastapi import Driver
from nonebot.log import logger
from nonebot.rule import to_me
from nonebot.typing import T_State

from ..plugin_config import plugin_config
from ..types import WeightConfig
from .api import (
    add_group_sub,
    auth,
    del_group_sub,
    get_global_conf,
    get_subs_info,
    get_target_name,
    get_weight_config,
    test,
    update_group_sub,
    update_weigth_config,
)
from .jwt import load_jwt
from .token_manager import token_manager as tm

URL_BASE = "/bison/"
GLOBAL_CONF_URL = f"{URL_BASE}api/global_conf"
AUTH_URL = f"{URL_BASE}api/auth"
SUBSCRIBE_URL = f"{URL_BASE}api/subs"
GET_TARGET_NAME_URL = f"{URL_BASE}api/target_name"
WEIGHT_URL = f"{URL_BASE}api/weight"
TEST_URL = f"{URL_BASE}test"

STATIC_PATH = (Path(__file__).parent / "dist").resolve()

sio = socketio.AsyncServer(async_mode="asgi", cors_allowed_origins="*")
socket_app = socketio.ASGIApp(sio, socketio_path="socket")


class SinglePageApplication(StaticFiles):
    def __init__(self, directory: os.PathLike, index="index.html"):
        self.index = index
        super().__init__(directory=directory, packages=None, html=True, check_dir=True)

    def lookup_path(self, path: str) -> tuple[str, Union[os.stat_result, None]]:
        full_path, stat_res = super().lookup_path(path)
        if stat_res is None:
            return super().lookup_path(self.index)
        return (full_path, stat_res)


def register_router_fastapi(driver: Driver, socketio):
    from fastapi import HTTPException, status
    from fastapi.param_functions import Depends
    from fastapi.security import OAuth2PasswordBearer

    oath_scheme = OAuth2PasswordBearer(tokenUrl="token")

    async def get_jwt_obj(token: str = Depends(oath_scheme)):
        obj = load_jwt(token)
        if not obj:
            raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED)
        return obj

    async def check_group_permission(
        groupNumber: int, token_obj: dict = Depends(get_jwt_obj)
    ):
        groups = token_obj["groups"]
        for group in groups:
            if int(groupNumber) == group["id"]:
                return
        raise HTTPException(status_code=status.HTTP_403_FORBIDDEN)

    async def check_is_superuser(token_obj: dict = Depends(get_jwt_obj)):
        if token_obj.get("type") != "admin":
            raise HTTPException(status_code=status.HTTP_403_FORBIDDEN)

    @dataclass
    class AddSubscribeReq:
        platformName: str
        target: str
        targetName: str
        cats: list[int]
        tags: list[str]

    app = driver.server_app
    static_path = STATIC_PATH
    app.get(TEST_URL)(test)
    app.get(GLOBAL_CONF_URL)(get_global_conf)
    app.get(AUTH_URL)(auth)

    @app.get(SUBSCRIBE_URL)
    async def subs(jwt_obj: dict = Depends(get_jwt_obj)):
        return await get_subs_info(jwt_obj)

    @app.get(GET_TARGET_NAME_URL)
    async def _get_target_name(
        platformName: str, target: str, jwt_obj: dict = Depends(get_jwt_obj)
    ):
        return await get_target_name(platformName, target, jwt_obj)

    @app.post(SUBSCRIBE_URL, dependencies=[Depends(check_group_permission)])
    async def _add_group_subs(groupNumber: int, req: AddSubscribeReq):
        return await add_group_sub(
            group_number=groupNumber,
            platform_name=req.platformName,
            target=req.target,
            target_name=req.targetName,
            cats=req.cats,
            tags=req.tags,
        )

    @app.patch(SUBSCRIBE_URL, dependencies=[Depends(check_group_permission)])
    async def _update_group_subs(groupNumber: int, req: AddSubscribeReq):
        return await update_group_sub(
            group_number=groupNumber,
            platform_name=req.platformName,
            target=req.target,
            target_name=req.targetName,
            cats=req.cats,
            tags=req.tags,
        )

    @app.delete(SUBSCRIBE_URL, dependencies=[Depends(check_group_permission)])
    async def _del_group_subs(groupNumber: int, target: str, platformName: str):
        return await del_group_sub(groupNumber, platformName, target)

    @app.get(WEIGHT_URL, dependencies=[Depends(check_is_superuser)])
    async def _get_weight_config():
        return await get_weight_config()

    @app.put(WEIGHT_URL, dependencies=[Depends(check_is_superuser)])
    async def _update_weight_config(platform_name: str, target: str, req: WeightConfig):
        return await update_weigth_config(platform_name, target, req)

    app.mount(URL_BASE, SinglePageApplication(directory=static_path), name="bison")


def init():
    driver = get_driver()
    if isinstance(driver, Driver):
        register_router_fastapi(driver, socket_app)
    else:
        logger.warning(f"Driver {driver.type} not supported")
        return
    host = str(driver.config.host)
    port = driver.config.port
    if host in ["0.0.0.0", "127.0.0.1"]:
        host = "localhost"
    logger.opt(colors=True).info(
        f"Nonebot test frontend will be running at: "
        f"<b><u>http://{host}:{port}{URL_BASE}</u></b>"
    )


if (STATIC_PATH / "index.html").exists():
    init()

    get_token = on_command("后台管理", rule=to_me(), priority=5)

    @get_token.handle()
    async def send_token(bot: "Bot", event: PrivateMessageEvent, state: T_State):
        token = tm.get_user_token((event.get_user_id(), event.sender.nickname))
        await get_token.finish(f"请访问: {plugin_config.bison_outer_url}auth/{token}")

    get_token.__help__name__ = "获取后台管理地址"
    get_token.__help__info__ = "获取管理bot后台的地址，该地址会" "在一段时间过后过期，请不要泄漏该地址"
else:
    logger.warning(
        "Frontend file not found, please compile it or use docker or pypi version"
    )



import calendar
from typing import Any, Optional

import feedparser
from bs4 import BeautifulSoup as bs
from httpx import AsyncClient

from ..post import Post
from ..types import RawPost, Target
from ..utils import http_client, scheduler
from .platform import NewMessage


class Rss(NewMessage):

    categories = {}
    enable_tag = False
    platform_name = "rss"
    name = "Rss"
    enabled = True
    is_common = True
    scheduler = scheduler("interval", {"seconds": 30})
    has_target = True

    @classmethod
    async def get_target_name(
        cls, client: AsyncClient, target: Target
    ) -> Optional[str]:
        async with http_client() as client:
            res = await client.get(target, timeout=10.0)
            feed = feedparser.parse(res.text)
            return feed["feed"]["title"]

    def get_date(self, post: RawPost) -> int:
        return calendar.timegm(post.published_parsed)

    def get_id(self, post: RawPost) -> Any:
        return post.id

    async def get_sub_list(self, target: Target) -> list[RawPost]:
        async with http_client() as client:
            res = await client.get(target, timeout=10.0)
            feed = feedparser.parse(res)
            entries = feed.entries
            for entry in entries:
                entry["_target_name"] = feed.feed.title
            return feed.entries

    async def parse(self, raw_post: RawPost) -> Post:
        text = raw_post.get("title", "") + "\n" if raw_post.get("title") else ""
        soup = bs(raw_post.description, "html.parser")
        text += soup.text.strip()
        pics = list(map(lambda x: x.attrs["src"], soup("img")))
        if raw_post.get("media_content"):
            for media in raw_post["media_content"]:
                if media.get("medium") == "image" and media.get("url"):
                    pics.append(media.get("url"))
        return Post(
            "rss",
            text=text,
            url=raw_post.link,
            pics=pics,
            target_name=raw_post["_target_name"],
        )



import json
from typing import Any, Optional

from bs4 import BeautifulSoup as bs
from httpx import AsyncClient
from nonebot.plugin import require

from ..post import Post
from ..types import Category, RawPost, Target
from ..utils.scheduler_config import SchedulerConfig
from .platform import CategoryNotSupport, NewMessage, StatusChange


class ArknightsSchedConf(SchedulerConfig):

    name = "arknights"
    schedule_type = "interval"
    schedule_setting = {"seconds": 30}


class Arknights(NewMessage):

    categories = {1: "游戏公告"}
    platform_name = "arknights"
    name = "明日方舟游戏信息"
    enable_tag = False
    enabled = True
    is_common = False
    scheduler = ArknightsSchedConf
    has_target = False

    @classmethod
    async def get_target_name(
        cls, client: AsyncClient, target: Target
    ) -> Optional[str]:
        return "明日方舟游戏信息"

    async def get_sub_list(self, _) -> list[RawPost]:
        raw_data = await self.client.get(
            "https://ak-conf.hypergryph.com/config/prod/announce_meta/IOS/announcement.meta.json"
        )
        return json.loads(raw_data.text)["announceList"]

    def get_id(self, post: RawPost) -> Any:
        return post["announceId"]

    def get_date(self, _: RawPost) -> None:
        return None

    def get_category(self, _) -> Category:
        return Category(1)

    async def parse(self, raw_post: RawPost) -> Post:
        announce_url = raw_post["webUrl"]
        text = ""
        raw_html = await self.client.get(announce_url)
        soup = bs(raw_html.text, "html.parser")
        pics = []
        if soup.find("div", class_="standerd-container"):
            # 图文
            require("nonebot_plugin_htmlrender")
            from nonebot_plugin_htmlrender import capture_element

            pic_data = await capture_element(
                announce_url,
                "div.main",
                viewport={"width": 320, "height": 6400},
                device_scale_factor=3,
            )
            # render = Render()
            # viewport = {"width": 320, "height": 6400, "deviceScaleFactor": 3}
            # pic_data = await render.render(
            #     announce_url, viewport=viewport, target="div.main"
            # )
            if pic_data:
                pics.append(pic_data)
            else:
                text = "图片渲染失败"
        elif pic := soup.find("img", class_="banner-image"):
            pics.append(pic["src"])  # type: ignore
        else:
            raise CategoryNotSupport()
        return Post(
            "arknights",
            text=text,
            url="",
            target_name="明日方舟游戏内公告",
            pics=pics,
            compress=True,
            override_use_pic=False,
        )


class AkVersion(StatusChange):

    categories = {2: "更新信息"}
    platform_name = "arknights"
    name = "明日方舟游戏信息"
    enable_tag = False
    enabled = True
    is_common = False
    scheduler = ArknightsSchedConf
    has_target = False

    @classmethod
    async def get_target_name(
        cls, client: AsyncClient, target: Target
    ) -> Optional[str]:
        return "明日方舟游戏信息"

    async def get_status(self, _):
        res_ver = await self.client.get(
            "https://ak-conf.hypergryph.com/config/prod/official/IOS/version"
        )
        res_preanounce = await self.client.get(
            "https://ak-conf.hypergryph.com/config/prod/announce_meta/IOS/preannouncement.meta.json"
        )
        res = res_ver.json()
        res.update(res_preanounce.json())
        return res

    def compare_status(self, _, old_status, new_status):
        res = []
        if (
            old_status.get("preAnnounceType") == 2
            and new_status.get("preAnnounceType") == 0
        ):
            res.append(
                Post("arknights", text="登录界面维护公告上线（大概是开始维护了)", target_name="明日方舟更新信息")
            )
        elif (
            old_status.get("preAnnounceType") == 0
            and new_status.get("preAnnounceType") == 2
        ):
            res.append(
                Post("arknights", text="登录界面维护公告下线（大概是开服了，冲！）", target_name="明日方舟更新信息")
            )
        if old_status.get("clientVersion") != new_status.get("clientVersion"):
            res.append(Post("arknights", text="游戏本体更新（大更新）", target_name="明日方舟更新信息"))
        if old_status.get("resVersion") != new_status.get("resVersion"):
            res.append(Post("arknights", text="游戏资源更新（小更新）", target_name="明日方舟更新信息"))
        return res

    def get_category(self, _):
        return Category(2)

    async def parse(self, raw_post):
        return raw_post


class MonsterSiren(NewMessage):

    categories = {3: "塞壬唱片新闻"}
    platform_name = "arknights"
    name = "明日方舟游戏信息"
    enable_tag = False
    enabled = True
    is_common = False
    scheduler = ArknightsSchedConf
    has_target = False

    @classmethod
    async def get_target_name(
        cls, client: AsyncClient, target: Target
    ) -> Optional[str]:
        return "明日方舟游戏信息"

    async def get_sub_list(self, _) -> list[RawPost]:
        raw_data = await self.client.get(
            "https://monster-siren.hypergryph.com/api/news"
        )
        return raw_data.json()["data"]["list"]

    def get_id(self, post: RawPost) -> Any:
        return post["cid"]

    def get_date(self, _) -> None:
        return None

    def get_category(self, _) -> Category:
        return Category(3)

    async def parse(self, raw_post: RawPost) -> Post:
        url = f'https://monster-siren.hypergryph.com/info/{raw_post["cid"]}'
        res = await self.client.get(
            f'https://monster-siren.hypergryph.com/api/news/{raw_post["cid"]}'
        )
        raw_data = res.json()
        content = raw_data["data"]["content"]
        content = content.replace("</p>", "</p>\n")
        soup = bs(content, "html.parser")
        imgs = list(map(lambda x: x["src"], soup("img")))
        text = f'{raw_post["title"]}\n{soup.text.strip()}'
        return Post(
            "monster-siren",
            text=text,
            pics=imgs,
            url=url,
            target_name="塞壬唱片新闻",
            compress=True,
            override_use_pic=False,
        )


class TerraHistoricusComic(NewMessage):

    categories = {4: "泰拉记事社漫画"}
    platform_name = "arknights"
    name = "明日方舟游戏信息"
    enable_tag = False
    enabled = True
    is_common = False
    scheduler = ArknightsSchedConf
    has_target = False

    @classmethod
    async def get_target_name(
        cls, client: AsyncClient, target: Target
    ) -> Optional[str]:
        return "明日方舟游戏信息"

    async def get_sub_list(self, _) -> list[RawPost]:
        raw_data = await self.client.get(
            "https://terra-historicus.hypergryph.com/api/recentUpdate"
        )
        return raw_data.json()["data"]

    def get_id(self, post: RawPost) -> Any:
        return f'{post["comicCid"]}/{post["episodeCid"]}'

    def get_date(self, _) -> None:
        return None

    def get_category(self, _) -> Category:
        return Category(4)

    async def parse(self, raw_post: RawPost) -> Post:
        url = f'https://terra-historicus.hypergryph.com/comic/{raw_post["comicCid"]}/episode/{raw_post["episodeCid"]}'
        return Post(
            "terra-historicus",
            text=f'{raw_post["title"]} - {raw_post["episodeShortTitle"]}',
            pics=[raw_post["coverUrl"]],
            url=url,
            target_name="泰拉记事社漫画",
            compress=True,
            override_use_pic=False,
        )



"""add constraint

Revision ID: c97c445e2bdb
Revises: 0571870f5222
Create Date: 2022-03-26 19:46:50.910721

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "c97c445e2bdb"
down_revision = "0571870f5222"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("subscribe", schema=None) as batch_op:
        batch_op.create_unique_constraint(
            "unique-subscribe-constraint", ["target_id", "user_id"]
        )

    with op.batch_alter_table("target", schema=None) as batch_op:
        batch_op.create_unique_constraint(
            "unique-target-constraint", ["target", "platform_name"]
        )

    with op.batch_alter_table("user", schema=None) as batch_op:
        batch_op.create_unique_constraint("unique-user-constraint", ["type", "uid"])

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("user", schema=None) as batch_op:
        batch_op.drop_constraint("unique-user-constraint", type_="unique")

    with op.batch_alter_table("target", schema=None) as batch_op:
        batch_op.drop_constraint("unique-target-constraint", type_="unique")

    with op.batch_alter_table("subscribe", schema=None) as batch_op:
        batch_op.drop_constraint("unique-subscribe-constraint", type_="unique")

    # ### end Alembic commands ###



from nonebot import get_driver
from nonebot.log import logger

from .config.config_legacy import start_up as legacy_db_startup
from .config.db import upgrade_db
from .scheduler.aps import start_scheduler
from .scheduler.manager import init_scheduler


@get_driver().on_startup
async def bootstrap():
    # legacy db
    legacy_db_startup()
    # new db
    await upgrade_db()
    # init scheduler
    await init_scheduler()
    # start scheduler
    start_scheduler()
    logger.info("nonebot-bison bootstrap done")



from collections import defaultdict
from importlib import import_module
from pathlib import Path
from pkgutil import iter_modules
from typing import DefaultDict, Type

from .platform import Platform, make_no_target_group

_package_dir = str(Path(__file__).resolve().parent)
for (_, module_name, _) in iter_modules([_package_dir]):
    import_module(f"{__name__}.{module_name}")


_platform_list: DefaultDict[str, list[Type[Platform]]] = defaultdict(list)
for _platform in Platform.registry:
    if not _platform.enabled:
        continue
    _platform_list[_platform.platform_name].append(_platform)

platform_manager: dict[str, Type[Platform]] = dict()
for name, platform_list in _platform_list.items():
    if len(platform_list) == 1:
        platform_manager[name] = platform_list[0]
    else:
        platform_manager[name] = make_no_target_group(platform_list)



from dataclasses import dataclass, field
from pathlib import Path
from typing import Optional

from nonebot.adapters.onebot.v11.message import Message, MessageSegment
from nonebot.log import logger
from nonebot.plugin import require

from .abstract_post import AbstractPost, BasePost


@dataclass
class _CustomPost(BasePost):

    message_segments: list[MessageSegment] = field(default_factory=list)
    css_path: Optional[str] = None  # 模板文件所用css路径

    async def generate_text_messages(self) -> list[MessageSegment]:
        return self.message_segments

    async def generate_pic_messages(self) -> list[MessageSegment]:
        require("nonebot_plugin_htmlrender")
        from nonebot_plugin_htmlrender import md_to_pic

        pic_bytes = await md_to_pic(md=self._generate_md(), css_path=self.css_path)
        return [MessageSegment.image(pic_bytes)]

    def _generate_md(self) -> str:
        md = ""

        for message_segment in self.message_segments:
            if message_segment.type == "text":
                md += "{}<br>".format(message_segment.data.get("text", ""))
            elif message_segment.type == "image":
                # 先尝试获取file的值，没有再尝试获取url的值，都没有则为空
                pic_res = message_segment.data.get("file") or message_segment.data.get(
                    "url", ""
                )
                if not pic_res:
                    logger.warning("无法获取到图片资源:MessageSegment.image中file/url字段均为空")
                else:
                    md += "![Image]({})\n".format(pic_res)
            else:
                logger.warning("custom_post不支持处理类型:{}".format(message_segment.type))
                continue

        return md


@dataclass
class CustomPost(_CustomPost, AbstractPost):
    """基于 markdown 语法的，自由度较高的推送内容格式

    简介:
    支持处理text/image两种MessageSegment,
    通过将text/image转换成对应的markdown语法以生成markdown文本。
    理论上text类型中可以直接使用markdown语法,例如`##第一章`。
    但会导致不启用`override_use_pic`时, 发送不会被渲染的纯文本消息。
    图片渲染最终由htmlrender执行。

    注意:
    每一个MessageSegment元素都会被解释为单独的一行

    可选参数:
    `override_use_pic`:是否覆盖`bison_use_pic`全局配置
    `compress`:将所有消息压缩为一条进行发送
    `extra_msg`:需要附带发送的额外消息

    成员函数:
    `generate_text_messages()`:负责生成文本消息
    `generate_pic_messages()`:负责生成图片消息
    """

    pass



import re
import time
from typing import Literal, Optional

import httpx
from bs4 import BeautifulSoup, NavigableString, Tag
from httpx import AsyncClient

from ..post import Post
from ..types import Category, RawPost, Target
from ..utils import scheduler
from .platform import CategoryNotSupport, NewMessage


def _format_text(rawtext: str, mode: int) -> str:
    """处理BeautifulSoup生成的string中奇怪的回车+连续空格
    mode 0:处理标题
    mode 1:处理版本资讯类推文
    mode 2:处理快讯类推文"""
    match mode:
        case 0:
            ftext = re.sub(r"\n\s*", " ", rawtext)
        case 1:
            ftext = re.sub(r"[\n\s*]", "", rawtext)
        case 2:
            ftext = re.sub(r"\r\n", "", rawtext)
    return ftext


def _stamp_date(rawdate: str) -> int:
    """将时间转化为时间戳yyyy-mm-dd->timestamp"""
    time_stamp = int(time.mktime(time.strptime(rawdate, "%Y-%m-%d")))
    return time_stamp


class McbbsNews(NewMessage):
    categories = {1: "Java版本资讯", 2: "基岩版本资讯", 3: "快讯", 4: "基岩快讯", 5: "周边消息"}
    enable_tag = False
    platform_name = "mcbbsnews"
    name = "MCBBS幻翼块讯"
    enabled = True
    is_common = False
    scheduler = scheduler("interval", {"hours": 1})
    has_target = False

    @classmethod
    async def get_target_name(
        cls, client: AsyncClient, target: Target
    ) -> Optional[str]:
        return cls.name

    async def get_sub_list(self, _: Target) -> list[RawPost]:
        url = "https://www.mcbbs.net/forum-news-1.html"
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/51.0.2704.63 Safari/537.36"
        }

        async with httpx.AsyncClient() as client:
            html = await client.get(url, headers=headers)
            soup = BeautifulSoup(html.text, "html.parser")
            raw_post_list = soup.find_all(
                "tbody", id=re.compile(r"normalthread_[0-9]*")
            )
            post_list = self._gen_post_list(raw_post_list)

        return post_list

    @staticmethod
    def _format_text(rawtext: str, mode: int) -> str:
        """处理BeautifulSoup生成的string中奇怪的回车+连续空格
        mode 0:处理标题
        mode 1:处理版本资讯类推文
        mode 2:处理快讯类推文"""
        if mode == 0:
            ftext = re.sub(r"\n\s*", " ", rawtext)
        elif mode == 1:
            ftext = re.sub(r"[\n\s*]", "", rawtext)
        elif mode == 2:
            ftext = re.sub(r"\r\n", "", rawtext)
        else:
            raise NotImplementedError
        return ftext

    @staticmethod
    def _stamp_date(rawdate: str) -> int:
        """将时间转化为时间戳yyyy-mm-dd->timestamp"""
        time_stamp = int(time.mktime(time.strptime(rawdate, "%Y-%m-%d")))
        return time_stamp

    def _gen_post_list(self, raw_post_list) -> list[RawPost]:
        """解析生成推文列表"""
        post_list = []
        for raw_post in raw_post_list:
            post = {}
            post["url"] = raw_post.find("a", class_="s xst")["href"]
            post["title"] = self._format_text(
                raw_post.find("a", class_="s xst").string, 0
            )
            post["category"] = raw_post.select("th em a")[0].string
            post["author"] = raw_post.select("td:nth-of-type(2) cite a")[0].string
            post["id"] = raw_post["id"]
            rawdate = (
                raw_post.select("td:nth-of-type(2) em span span")[0]["title"]
                if raw_post.select("td:nth-of-type(2) em span span")
                else raw_post.select("td:nth-of-type(2) em span")[0].string
            )
            post["date"] = self._stamp_date(rawdate)
            post_list.append(post)
        return post_list

    def get_id(self, post: RawPost) -> str:
        return post["id"]

    def get_date(self, post: RawPost) -> int:
        # 获取datetime精度只到日期，故暂时舍弃
        # return post["date"]
        return None

    def get_category(self, post: RawPost) -> Category:
        if post["category"] == "Java版本资讯":
            return Category(1)
        elif post["category"] == "基岩版本资讯":
            return Category(2)
        else:
            raise CategoryNotSupport("McbbsNews订阅暂不支持 `{}".format(post["category"]))

    @staticmethod
    def _check_str_chinese(check_str: str) -> bool:
        """检测字符串是否含有中文（有一个就算）"""
        for ch in check_str:
            if "\u4e00" <= ch <= "\u9fff":
                return True
        return False

    def _news_parser(self, raw_text: str, news_type: Literal["Java版本资讯", "基岩版本资讯"]):
        """提取Java/Bedrock版本资讯的推送消息"""
        raw_soup = BeautifulSoup(raw_text.replace("<br />", ""), "html.parser")
        # 获取头图
        if news_type == "Java版本资讯":
            # 获取头图
            pic_tag = raw_soup.find(
                "img", file=re.compile(r"https://www.minecraft.net/\S*header.jpg")
            )
            pic_url: list[str] = (
                [pic_tag.get("src", pic_tag.get("file"))] if pic_tag else []
            )
            # 获取blockquote标签下的内容
            soup = raw_soup.find(
                "td", id=re.compile(r"postmessage_[0-9]*")
            ).blockquote.blockquote
        elif news_type == "基岩版本资讯":
            # 获取头图
            pic_tag_0 = raw_soup.find(
                "img", file=re.compile(r"https://www.minecraft.net/\S*header.jpg")
            )
            pic_tag_1 = raw_soup.find(
                "img",
                file=re.compile(r"https://feedback.minecraft.net/\S*beta\S*.jpg"),
            )
            pic_url: list[str] = [
                pic_tag_0.get("src", pic_tag_0.get("file")) if pic_tag_0 else None,
                pic_tag_1.get("src", pic_tag_1.get("file")) if pic_tag_1 else None,
            ]
            # 获取blockquote标签下的内容
            soup = (
                raw_soup.find("td", id=re.compile(r"postmessage_[0-9]*"))
                .select("blockquote:nth-of-type(2)")[0]
                .blockquote
            )
        else:
            raise CategoryNotSupport(f"该函数不支持处理{news_type}")

        # 通用步骤
        # 删除无用的div和span段内容
        for del_tag in soup.find_all(["div", "span"]):
            del_tag.extract()
        # 进一步删除无用尾部
        # orig_info=soup.select("blockquote > strong")
        # orig_info[0].extract()
        # 展开所有的a,u和strong标签,展开ul,font标签里的font标签
        for unwrap_tag in soup.find_all(["a", "strong", "u", "ul", "font"]):
            if unwrap_tag.name in ["a", "strong", "u"]:  # 展开所有的a,u和strong标签
                unwrap_tag.unwrap()
            elif unwrap_tag.name in ["ul", "font"]:  # 展开ul,font里的font标签
                for font_tag in unwrap_tag.find_all("font"):
                    font_tag.unwrap()

        # 获取所有的中文句子
        post_text = ""
        last_is_empty_line = True
        for element in soup.contents:
            if isinstance(element, Tag):
                if element.name == "font":
                    text = ""
                    for sub in element.contents:
                        if isinstance(sub, NavigableString):
                            text += sub
                    if self._check_str_chinese(text):
                        post_text += "{}\n".format(self._format_text(text, 1))
                        last_is_empty_line = False
                elif element.name == "ul":
                    for li_tag in element.find_all("li"):
                        text = ""
                        for sub in li_tag.contents:
                            if isinstance(sub, NavigableString):
                                text += sub
                        if self._check_str_chinese(text):
                            post_text += "{}\n".format(self._format_text(text, 1))
                            last_is_empty_line = False
                else:
                    continue
            elif isinstance(element, NavigableString):
                if str(element) == "\n":
                    if not last_is_empty_line:
                        post_text += "\n"
                    last_is_empty_line = True
                else:
                    post_text += "{}\n".format(self._format_text(element, 1))
                    last_is_empty_line = False
            else:
                continue
        return post_text, pic_url

    def _express_parser(self, raw_text: str, news_type: Literal["快讯", "基岩快讯", "周边消息"]):
        """提取快讯/基岩快讯/周边消息的推送消息"""
        raw_soup = BeautifulSoup(raw_text.replace("<br />", ""), "html.parser")
        # 获取原始推文内容
        soup = raw_soup.find("td", id=re.compile(r"postmessage_[0-9]*"))
        if tag := soup.find("ignore_js_op"):
            tag.extract()
        # 获取所有图片
        pic_urls = []
        for img_tag in soup.find_all("img"):
            pic_url = img_tag.get("file") or img_tag.get("src")
            pic_urls.append(pic_url)
        # 验证是否有blockquote标签
        has_bolockquote = soup.find("blockquote")
        # 删除无用的span,div段内容
        for del_tag in soup.find_all("i"):
            del_tag.extract()
        if extag := soup.find(class_="attach_nopermission attach_tips"):
            extag.extract()
        # 展开所有的a,strong标签
        for unwrap_tag in soup.find_all(["a", "strong"]):
            unwrap_tag.unwrap()
        # 展开blockquote标签里的blockquote标签
        for b_tag in soup.find_all("blockquote"):
            for unwrap_tag in b_tag.find_all("blockquote"):
                unwrap_tag.unwrap()
        # 获取推文
        text = ""
        if has_bolockquote:
            for post in soup.find_all("blockquote"):
                # post.font.unwrap()
                for string in post.stripped_strings:
                    text += "{}\n".format(string)
        else:
            for string in soup.stripped_strings:
                text += "{}\n".format(string)
        ftext = self._format_text(text, 2)
        return ftext, pic_urls

    async def parse(self, raw_post: RawPost) -> Post:
        """获取并分配正式推文交由相应的函数解析"""
        post_url = "https://www.mcbbs.net/{}".format(raw_post["url"])
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/51.0.2704.63 Safari/537.36"
        }

        async with httpx.AsyncClient() as client:
            html = await client.get(post_url, headers=headers)

        if raw_post["category"] in ["Java版本资讯", "基岩版本资讯"]:
            # 事先删除不需要的尾部
            raw_text = re.sub(r"【本文排版借助了：[\s\S]*】", "", html.text)
            text, pic_urls = self._news_parser(raw_text, raw_post["category"])
        elif raw_post["category"] in ["快讯", "基岩快讯", "周边消息"]:
            text, pic_urls = self._express_parser(html.text, raw_post["category"])
        else:
            raise CategoryNotSupport("McbbsNews订阅暂不支持 `{}".format(raw_post["category"]))

        return Post(
            self.name,
            text="{}\n\n{}".format(raw_post["title"], text),
            url=post_url,
            pics=pic_urls,
            target_name=raw_post["category"],
        )



from typing import Literal, Type

from httpx import AsyncClient

from ..types import Target
from .http import http_client


class SchedulerConfig:

    schedule_type: Literal["date", "interval", "cron"]
    schedule_setting: dict
    name: str

    def __str__(self):
        return f"[{self.name}]-{self.name}-{self.schedule_setting}"

    def __init__(self):
        self.default_http_client = http_client()

    async def get_client(self, target: Target) -> AsyncClient:
        return self.default_http_client

    async def get_query_name_client(self) -> AsyncClient:
        return self.default_http_client


def scheduler(
    schedule_type: Literal["date", "interval", "cron"], schedule_setting: dict
) -> Type[SchedulerConfig]:
    return type(
        "AnonymousScheduleConfig",
        (SchedulerConfig,),
        {
            "schedule_type": schedule_type,
            "schedule_setting": schedule_setting,
        },
    )



import logging

import nonebot
from apscheduler.schedulers.asyncio import AsyncIOScheduler
from nonebot import get_driver
from nonebot.adapters.onebot.v11.bot import Bot
from nonebot.log import LoguruHandler, logger

from .config import config
from .platform import platform_manager
from .plugin_config import plugin_config
from .send import do_send_msgs, send_msgs
from .types import UserSubInfo

scheduler = AsyncIOScheduler(timezone="Asia/Shanghai")


@get_driver().on_startup
async def _start():
    for platform_name, platform in platform_manager.items():
        if platform.schedule_type in ["cron", "interval", "date"]:
            logger.info(
                f"start scheduler for {platform_name} with {platform.schedule_type} {platform.schedule_kw}"
            )
            scheduler.add_job(
                fetch_and_send,
                platform.schedule_type,
                **platform.schedule_kw,
                args=(platform_name,),
            )

    scheduler.configure({"apscheduler.timezone": "Asia/Shanghai"})
    scheduler.start()


# get_driver().on_startup(_start)


async def fetch_and_send(target_type: str):
    target = config.get_next_target(target_type)
    if not target:
        return
    logger.debug(
        "try to fecth new posts from {}, target: {}".format(target_type, target)
    )
    send_user_list = config.target_user_cache[target_type][target]
    send_userinfo_list = list(
        map(
            lambda user: UserSubInfo(
                user,
                lambda target: config.get_sub_category(
                    target_type, target, user.user_type, user.user
                ),
                lambda target: config.get_sub_tags(
                    target_type, target, user.user_type, user.user
                ),
            ),
            send_user_list,
        )
    )
    to_send = await platform_manager[target_type].do_fetch_new_post(
        target, send_userinfo_list
    )
    if not to_send:
        return
    bot = nonebot.get_bot()
    assert isinstance(bot, Bot)
    for user, send_list in to_send:
        for send_post in send_list:
            logger.info("send to {}: {}".format(user, send_post))
            if not bot:
                logger.warning("no bot connected")
            else:
                await send_msgs(
                    bot, user.user, user.user_type, await send_post.generate_messages()
                )


class CustomLogHandler(LoguruHandler):
    def filter(self, record: logging.LogRecord):
        return record.msg != (
            'Execution of job "%s" '
            "skipped: maximum number of running instances reached (%d)"
        )


if plugin_config.bison_use_queue:
    scheduler.add_job(do_send_msgs, "interval", seconds=0.3, coalesce=True)

    aps_logger = logging.getLogger("apscheduler")
    aps_logger.setLevel(30)
    aps_logger.handlers.clear()
    aps_logger.addHandler(CustomLogHandler())



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src/plugins'}

packages = \
['nonebot_bison',
 'nonebot_bison.admin_page',
 'nonebot_bison.config',
 'nonebot_bison.config.migrate',
 'nonebot_bison.config.migrate.versions',
 'nonebot_bison.platform',
 'nonebot_bison.post',
 'nonebot_bison.scheduler',
 'nonebot_bison.utils']

package_data = \
{'': ['*'],
 'nonebot_bison.admin_page': ['dist/*',
                              'dist/static/css/*',
                              'dist/static/js/*'],
 'nonebot_bison.post': ['templates/*']}

install_requires = \
['aiofiles>=0.8.0,<0.9.0',
 'alembic>=1.7.6,<2.0.0',
 'apscheduler>=3.7.0,<4.0.0',
 'bs4>=0.0.1,<0.0.2',
 'expiringdict>=1.2.1,<2.0.0',
 'feedparser>=6.0.2,<7.0.0',
 'httpx>=0.16.1',
 'nonebot-adapter-onebot>=2.0.0-beta.1,<3.0.0',
 'nonebot-plugin-datastore>=0.4.0,<0.5.0',
 'nonebot-plugin-htmlrender>=0.2.0',
 'nonebot2>=2.0.0-rc.1,<3.0.0',
 'pillow>=8.1,<10.0',
 'pyjwt>=2.1.0,<3.0.0',
 'python-socketio>=5.4.0,<6.0.0',
 'tinydb>=4.3.0,<5.0.0']

setup_kwargs = {
    'name': 'nonebot-bison',
    'version': '0.6.2',
    'description': 'Subscribe message from social medias',
    'long_description': '<div align="center">\n<h1>Bison </br>通用订阅推送插件</h1>\n\n[![pypi](https://badgen.net/pypi/v/nonebot-bison)](https://pypi.org/project/nonebot-bison/)\n[![license](https://img.shields.io/github/license/felinae98/nonebot-bison)](https://github.com/felinae98/nonebot-bison/blob/main/LICENSE)\n[![action](https://img.shields.io/github/workflow/status/felinae98/nonebot-bison/test-build)](https://github.com/felinae98/nonebot-bison/actions/workflows/main.yml)\n[![docker](https://img.shields.io/docker/image-size/felinae98/nonebot-bison)](https://hub.docker.com/r/felinae98/nonebot-bison)\n[![codecov](https://codecov.io/gh/felinae98/nonebot-bison/branch/main/graph/badge.svg?token=QCFIODJOOA)](https://codecov.io/gh/felinae98/nonebot-bison)\n[![qq group](https://img.shields.io/badge/QQ%E7%BE%A4-868610060-orange)](https://qm.qq.com/cgi-bin/qm/qr?k=pXYMGB_e8b6so3QTqgeV6lkKDtEeYE4f&jump_from=webapi)\n\n[文档](https://nonebot-bison.vercel.app)|[开发文档](https://nonebot-bison.vercel.app/dev)\n\n</div>\n\n## 简介\n\n一款自动爬取各种站点，社交平台更新动态，并将信息推送到 QQ 的机器人。\n基于 [`NoneBot2`](https://github.com/nonebot/nonebot2) 开发（诞生于明日方舟的蹲饼活动）\n\n<details>\n<summary>本项目原名原名nonebot-hk-reporter</summary>\n\n寓意本 Bot 要做全世界跑的最快的搬运机器人，后因名字过于暴力改名\n\n</details>\n本项目名称来源于明日方舟角色拜松——一名龙门的信使，曾经骑自行车追上骑摩托车的德克萨斯\n\n支持的平台：\n\n- 微博\n- Bilibili\n- Bilibili 直播\n- RSS\n- 明日方舟\n- 网易云音乐\n- FF14\n- mcbbs 幻翼块讯\n\n## 功能\n\n- [x] 定时爬取指定网站\n- [x] 通过图片发送文本，防止风控\n- [x] 使用队列限制发送频率\n- [x] 使用网页后台管理 Bot 订阅\n- [ ] 使用可以设置权重的调度器按时间调节不同账号的权重\n\n## 使用方法\n\n**!!注意，如果要使用后台管理功能请使用 pypi 版本或者 docker 版本，如果直接 clone 源代码\n需要按下面方式进行 build**\n\n```bash\ncd ./admin-frontend\npnpm && pnpm run build\n```\n\n可以使用 Docker，docker-compose，作为插件安装在 nonebot 中，或者直接运行\n\n在群里 at Bot 或者直接私聊 Bot “添加订阅”，按照提示输入需要订阅的账号，就可以愉快接收消息了。\n\n参考[文档](https://nonebot-bison.vercel.app/usage/#%E4%BD%BF%E7%94%A8)\n\n## FAQ\n\n1. 报错`TypeError: \'type\' object is not subscriptable`  \n   本项目使用了 Python 3.10 的语法，请将 Python 版本升级到 3.10 及以上，推荐使用 docker 部署\n2. bot 不理我  \n   请确认自己是群主或者管理员，并且检查`COMMAND_START`环境变量是否设为`[""]`\n   或者按照`COMMAND_START`中的设置添加命令前缀，例：\n   `COMMAND_START=["/"]`则应发送`/添加订阅`\n3. 微博漏订阅了\n   微博更新了新的风控措施，某些含有某些关键词的微博会获取不到。\n4. 无法使用后台管理页面\n   1. 确认自己正确配置了 nonebot 的端口，如果在远程或容器外访问网页请确保`HOST=0.0.0.0`\n   2. 确认自己的云服务器的防火墙配置正确\n   3. 确认自己使用了正确的方法安装插件\n\n## 参与开发\n\n欢迎各种 PR，参与开发本插件很简单，只需要对相应平台完成几个接口的编写就行。你只需要一点简单的爬虫知识就行。\n\n如果对整体框架有任何意见或者建议，欢迎 issue。\n\n## 鸣谢\n\n- [`go-cqhttp`](https://github.com/Mrs4s/go-cqhttp)：简单又完善的 cqhttp 实现\n- [`NoneBot2`](https://github.com/nonebot/nonebot2)：超好用的开发框架\n- [`HarukaBot`](https://github.com/SK-415/HarukaBot/): 借鉴了大体的实现思路\n- [`rsshub`](https://github.com/DIYgod/RSSHub)：提供了大量的 api\n\n## License\n\nMIT\n',
    'author': 'felinae98',
    'author_email': 'felinae225@qq.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/felinae98/nonebot-bison',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0.0',
}


setup(**setup_kwargs)



from dataclasses import dataclass
from typing import Optional, Type

import nonebot
from nonebot.adapters.onebot.v11.bot import Bot
from nonebot.log import logger

from ..config import config
from ..platform import platform_manager
from ..send import send_msgs
from ..types import Target
from ..utils import SchedulerConfig
from .aps import aps


@dataclass
class Schedulable:
    platform_name: str
    target: Target
    current_weight: int


class Scheduler:

    schedulable_list: list[Schedulable]

    def __init__(
        self,
        scheduler_config: Type[SchedulerConfig],
        schedulables: list[tuple[str, Target]],
        platform_name_list: list[str],
    ):
        self.name = scheduler_config.name
        if not scheduler_config:
            logger.error(f"scheduler config [{self.name}] not found, exiting")
            raise RuntimeError(f"{self.name} not found")
        self.scheduler_config = scheduler_config
        self.scheduler_config_obj = self.scheduler_config()
        self.schedulable_list = []
        for platform_name, target in schedulables:
            self.schedulable_list.append(
                Schedulable(
                    platform_name=platform_name, target=target, current_weight=0
                )
            )
        self.platform_name_list = platform_name_list
        self.pre_weight_val = 0  # 轮调度中“本轮”增加权重和的初值
        logger.info(
            f"register scheduler for {self.name} with {self.scheduler_config.schedule_type} {self.scheduler_config.schedule_setting}"
        )
        aps.add_job(
            self.exec_fetch,
            self.scheduler_config.schedule_type,
            **self.scheduler_config.schedule_setting,
        )

    async def get_next_schedulable(self) -> Optional[Schedulable]:
        if not self.schedulable_list:
            return None
        cur_weight = await config.get_current_weight_val(self.platform_name_list)
        weight_sum = self.pre_weight_val
        self.pre_weight_val = 0
        cur_max_schedulable = None
        for schedulable in self.schedulable_list:
            schedulable.current_weight += cur_weight[
                f"{schedulable.platform_name}-{schedulable.target}"
            ]
            weight_sum += cur_weight[
                f"{schedulable.platform_name}-{schedulable.target}"
            ]
            if (
                not cur_max_schedulable
                or cur_max_schedulable.current_weight < schedulable.current_weight
            ):
                cur_max_schedulable = schedulable
        assert cur_max_schedulable
        cur_max_schedulable.current_weight -= weight_sum
        return cur_max_schedulable

    async def exec_fetch(self):
        if not (schedulable := await self.get_next_schedulable()):
            return
        logger.debug(
            f"scheduler {self.name} fetching next target: [{schedulable.platform_name}]{schedulable.target}"
        )
        send_userinfo_list = await config.get_platform_target_subscribers(
            schedulable.platform_name, schedulable.target
        )
        platform_obj = platform_manager[schedulable.platform_name](
            await self.scheduler_config_obj.get_client(schedulable.target)
        )
        to_send = await platform_obj.do_fetch_new_post(
            schedulable.target, send_userinfo_list
        )
        if not to_send:
            return
        bot = nonebot.get_bot()
        assert isinstance(bot, Bot)
        for user, send_list in to_send:
            for send_post in send_list:
                logger.info("send to {}: {}".format(user, send_post))
                if not bot:
                    logger.warning("no bot connected")
                else:
                    await send_msgs(
                        bot,
                        user.user,
                        user.user_type,
                        await send_post.generate_messages(),
                    )

    def insert_new_schedulable(self, platform_name: str, target: Target):
        self.pre_weight_val += 1000
        self.schedulable_list.append(Schedulable(platform_name, target, 1000))
        logger.info(
            f"insert [{platform_name}]{target} to Schduler({self.scheduler_config.name})"
        )

    def delete_schedulable(self, platform_name, target: Target):
        if not self.schedulable_list:
            return
        to_find_idx = None
        for idx, schedulable in enumerate(self.schedulable_list):
            if (
                schedulable.platform_name == platform_name
                and schedulable.target == target
            ):
                to_find_idx = idx
                break
        if to_find_idx is not None:
            deleted_schdulable = self.schedulable_list.pop(to_find_idx)
            self.pre_weight_val -= deleted_schdulable.current_weight
        return
